@@ -17,15 +17,15 @@
 Image Cache Management API
 """
 
+from oslo_log import log as logging
 import routes
 
 from glance.api import cached_images
 from glance.common import wsgi
-from glance.openstack.common import gettextutils
-import glance.openstack.common.log as logging
+from glance import i18n
 
 LOG = logging.getLogger(__name__)
-_LI = gettextutils._LI
+_LI = i18n._LI
 
 
 class CacheManageFilter(wsgi.Middleware):
