@@ -19,16 +19,19 @@
 
 import copy
 
-import eventlet
 import glance_store as store
 import glance_store.location
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import excutils
+from oslo_utils import strutils
 from webob.exc import HTTPBadRequest
 from webob.exc import HTTPConflict
 from webob.exc import HTTPForbidden
 from webob.exc import HTTPMethodNotAllowed
 from webob.exc import HTTPNotFound
 from webob.exc import HTTPRequestEntityTooLarge
+from webob.exc import HTTPServiceUnavailable
 from webob import Response
 
 from glance.api import common
@@ -42,17 +45,15 @@ from glance.common import property_utils
 from glance.common import store_utils
 from glance.common import utils
 from glance.common import wsgi
-from glance.i18n import _LE
+from glance import i18n
 from glance import notifier
-from glance.openstack.common import excutils
-from glance.openstack.common import gettextutils
-import glance.openstack.common.log as logging
-from glance.openstack.common import strutils
 import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
-_LI = gettextutils._LI
-_LW = gettextutils._LW
+_ = i18n._
+_LE = i18n._LE
+_LI = i18n._LI
+_LW = i18n._LW
 SUPPORTED_PARAMS = glance.api.v1.SUPPORTED_PARAMS
 SUPPORTED_FILTERS = glance.api.v1.SUPPORTED_FILTERS
 ACTIVE_IMMUTABLE = glance.api.v1.ACTIVE_IMMUTABLE
@@ -140,7 +141,6 @@ class Controller(controller.BaseController):
         self.notifier = notifier.Notifier()
         registry.configure_registry_client()
         self.policy = policy.Enforcer()
-        self.pool = eventlet.GreenPool(size=1024)
         if property_utils.is_property_protection_enabled():
             self.prop_enforcer = property_utils.PropertyRules(self.policy)
         else:
@@ -178,7 +178,7 @@ class Controller(controller.BaseController):
                      "image properties. Attempted: %(num)s, Maximum: "
                      "%(quota)s") % {'num': len(props),
                                      'quota': CONF.image_property_quota})
-            LOG.info(msg)
+            LOG.warn(msg)
             raise HTTPRequestEntityTooLarge(explanation=msg,
                                             request=req,
                                             content_type="text/plain")
@@ -196,8 +196,8 @@ class Controller(controller.BaseController):
             for key in create_props:
                 if (self.prop_enforcer.check_property_rules(
                         key, 'create', req.context) is False):
-                    msg = "Property '%s' is protected" % key
-                    LOG.debug(msg)
+                    msg = _("Property '%s' is protected") % key
+                    LOG.warn(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
                                         content_type="text/plain")
@@ -240,8 +240,8 @@ class Controller(controller.BaseController):
                         key, 'update', req.context) is False and
                         image_meta['properties'][key] !=
                         orig_meta['properties'][key]) or not has_read):
-                    msg = "Property '%s' is protected" % key
-                    LOG.debug(msg)
+                    msg = _("Property '%s' is protected") % key
+                    LOG.warn(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
                                         content_type="text/plain")
@@ -270,12 +270,12 @@ class Controller(controller.BaseController):
                         key, 'read', req.context) is False):
                     # NOTE(bourke): if read protected, re-add to image_meta to
                     # prevent deletion
-                    image_meta['properties'][key] = \
-                        orig_meta['properties'][key]
+                    image_meta['properties'][key] = orig_meta[
+                        'properties'][key]
                 elif (self.prop_enforcer.check_property_rules(
                         key, 'delete', req.context) is False):
-                    msg = "Property '%s' is protected" % key
-                    LOG.debug(msg)
+                    msg = _("Property '%s' is protected") % key
+                    LOG.warn(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
                                         content_type="text/plain")
@@ -416,7 +416,7 @@ class Controller(controller.BaseController):
     @staticmethod
     def _validate_source(source, req):
         """
-        To validate if external sources (as specified via the location
+        Validate if external sources (as specified via the location
         or copy-from headers) are supported. Otherwise we reject
         with 400 "Bad Request".
         """
@@ -424,8 +424,8 @@ class Controller(controller.BaseController):
             if store_utils.validate_external_location(source):
                 return source
             else:
-                msg = _("External source are not supported: '%s'") % source
-                LOG.debug(msg)
+                msg = _("External sources are not supported: '%s'") % source
+                LOG.warn(msg)
                 raise HTTPBadRequest(explanation=msg,
                                      request=req,
                                      content_type="text/plain")
@@ -453,8 +453,14 @@ class Controller(controller.BaseController):
 
             image_data, image_size = src_store.get(loc, context=context)
 
+        except store.RemoteServiceUnavailable as e:
+            raise HTTPServiceUnavailable(explanation=e.msg)
         except store.NotFound as e:
             raise HTTPNotFound(explanation=e.msg)
+        except (store.StoreGetNotSupported,
+                store.StoreRandomGetNotSupported,
+                store.UnknownScheme) as e:
+            raise HTTPBadRequest(explanation=e.msg)
         image_size = int(image_size) if image_size else None
         return image_data, image_size
 
@@ -472,7 +478,7 @@ class Controller(controller.BaseController):
         self._enforce(req, 'get_image')
 
         try:
-            image_meta = self.get_active_image_meta_or_404(req, id)
+            image_meta = self.get_active_image_meta_or_error(req, id)
         except HTTPNotFound:
             # provision for backward-compatibility breaking issue
             # catch the 404 exception and raise it after enforcing
@@ -514,8 +520,8 @@ class Controller(controller.BaseController):
         location = self._external_source(image_meta, req)
         scheme = image_meta.get('store')
         if scheme and scheme not in store.get_known_schemes():
-            msg = "Required store %s is invalid" % scheme
-            LOG.debug(msg)
+            msg = _("Required store %s is invalid") % scheme
+            LOG.warn(msg)
             raise HTTPBadRequest(explanation=msg,
                                  content_type='text/plain')
 
@@ -525,8 +531,8 @@ class Controller(controller.BaseController):
         if location:
             try:
                 backend = store.get_store_from_location(location)
-            except store.BadStoreUri:
-                msg = "Invalid location %s" % location
+            except (store.UnknownScheme, store.BadStoreUri):
+                msg = _("Invalid location %s") % location
                 LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg,
                                      request=req,
@@ -549,23 +555,22 @@ class Controller(controller.BaseController):
             self.notifier.info("image.create", redact_loc(image_meta))
             return image_meta
         except exception.Duplicate:
-            msg = ("An image with identifier %s already exists" %
+            msg = (_("An image with identifier %s already exists") %
                    image_meta['id'])
-            LOG.debug(msg)
+            LOG.warn(msg)
             raise HTTPConflict(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Invalid as e:
             msg = (_("Failed to reserve image. Got error: %s") %
                    utils.exception_to_str(e))
-            for line in msg.split('\n'):
-                LOG.debug(line)
+            LOG.exception(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.Forbidden:
-            msg = "Forbidden to reserve image."
-            LOG.debug(msg)
+            msg = _("Forbidden to reserve image.")
+            LOG.warn(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
@@ -608,8 +613,8 @@ class Controller(controller.BaseController):
                 req.get_content_type(('application/octet-stream',))
             except exception.InvalidContentType:
                 upload_utils.safe_kill(req, image_meta['id'], 'queued')
-                msg = "Content-Type must be application/octet-stream"
-                LOG.debug(msg)
+                msg = _("Content-Type must be application/octet-stream")
+                LOG.warn(msg)
                 raise HTTPBadRequest(explanation=msg)
 
             image_data = req.body_file
@@ -664,9 +669,9 @@ class Controller(controller.BaseController):
                           {'id': image_id, 'location': image_meta['location']})
                 upload_utils.initiate_deletion(req, location_data, image_id)
         except exception.Invalid as e:
-            msg = ("Failed to activate image. Got error: %s" %
+            msg = (_("Failed to activate image. Got error: %s") %
                    utils.exception_to_str(e))
-            LOG.debug(msg)
+            LOG.warn(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
@@ -715,8 +720,19 @@ class Controller(controller.BaseController):
         try:
             return (image_meta.get('size', 0) or
                     store.get_size_from_backend(location, context=context))
-        except (store.NotFound, store.BadStoreUri) as e:
-            LOG.debug(e)
+        except store.NotFound as e:
+            # NOTE(rajesht): The exception is logged as debug message because
+            # the image is located at third-party server and it has nothing to
+            # do with glance. If log.exception is used here, in that case the
+            # log file might be flooded with exception log messages if
+            # malicious user keeps on trying image-create using non-existent
+            # location url. Used log.debug because administrator can
+            # disable debug logs.
+            LOG.debug(utils.exception_to_str(e))
+            raise HTTPNotFound(explanation=e.msg, content_type="text/plain")
+        except (store.UnknownScheme, store.BadStoreUri) as e:
+            # NOTE(rajesht): See above note of store.NotFound
+            LOG.debug(utils.exception_to_str(e))
             raise HTTPBadRequest(explanation=e.msg, content_type="text/plain")
 
     def _handle_source(self, req, image_id, image_meta, image_data):
@@ -724,8 +740,8 @@ class Controller(controller.BaseController):
         location = image_meta.get('location')
         sources = filter(lambda x: x, (copy_from, location, image_data))
         if len(sources) >= 2:
-            msg = "It's invalid to provide multiple image sources."
-            LOG.debug(msg)
+            msg = _("It's invalid to provide multiple image sources.")
+            LOG.warn(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
@@ -737,7 +753,8 @@ class Controller(controller.BaseController):
         elif copy_from:
             msg = _LI('Triggering asynchronous copy from external source')
             LOG.info(msg)
-            self.pool.spawn_n(self._upload_and_activate, req, image_meta)
+            pool = common.get_thread_pool("copy_from_eventlet_pool")
+            pool.spawn_n(self._upload_and_activate, req, image_meta)
         else:
             if location:
                 self._validate_image_for_activation(req, image_id, image_meta)
@@ -756,12 +773,12 @@ class Controller(controller.BaseController):
                     # size provided by the client will be used as-is.
                     if (image_size_store and
                             image_size_store != image_size_meta):
-                        msg = ("Provided image size must match the stored "
-                               "image size. (provided size: %(ps)d, "
-                               "stored size: %(ss)d)" % {
-                                   "ps": image_size_meta,
-                                   "ss": image_size_store})
-                        LOG.debug(msg)
+                        msg = (_("Provided image size must match the stored"
+                                 " image size. (provided size: %(ps)d, "
+                                 "stored size: %(ss)d)") %
+                               {"ps": image_size_meta,
+                                "ss": image_size_store})
+                        LOG.warn(msg)
                         raise HTTPConflict(explanation=msg,
                                            request=req,
                                            content_type="text/plain")
@@ -934,8 +951,8 @@ class Controller(controller.BaseController):
                 self.update_store_acls(req, id, orig_or_updated_loc,
                                        public=is_public)
             except store.BadStoreUri:
-                msg = "Invalid location %s" % location
-                LOG.debug(msg)
+                msg = _("Invalid location: %s") % location
+                LOG.warn(msg)
                 raise HTTPBadRequest(explanation=msg,
                                      request=req,
                                      content_type="text/plain")
@@ -981,31 +998,29 @@ class Controller(controller.BaseController):
                                                  image_data)
 
         except exception.Invalid as e:
-            msg = ("Failed to update image metadata. Got error: %s" %
+            msg = (_("Failed to update image metadata. Got error: %s") %
                    utils.exception_to_str(e))
-            LOG.debug(msg)
+            LOG.warn(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.NotFound as e:
             msg = (_("Failed to find image to update: %s") %
                    utils.exception_to_str(e))
-            for line in msg.split('\n'):
-                LOG.info(line)
+            LOG.warn(msg)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
             msg = (_("Forbidden to update image: %s") %
                    utils.exception_to_str(e))
-            for line in msg.split('\n'):
-                LOG.info(line)
+            LOG.warn(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
         except (exception.Conflict, exception.Duplicate) as e:
-            LOG.info(utils.exception_to_str(e))
-            raise HTTPConflict(body='Image operation conflicts',
+            LOG.warn(utils.exception_to_str(e))
+            raise HTTPConflict(body=_('Image operation conflicts'),
                                request=req,
                                content_type='text/plain')
         else:
@@ -1036,20 +1051,22 @@ class Controller(controller.BaseController):
 
         image = self.get_image_meta_or_404(req, id)
         if image['protected']:
-            msg = "Image is protected"
-            LOG.debug(msg)
+            msg = _("Image is protected")
+            LOG.warn(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
 
         if image['status'] == 'pending_delete':
-            msg = "Forbidden to delete a %s image." % image['status']
-            LOG.debug(msg)
-            raise HTTPForbidden(explanation=msg, request=req,
+            msg = (_("Forbidden to delete a %s image.") %
+                   image['status'])
+            LOG.warn(msg)
+            raise HTTPForbidden(explanation=msg,
+                                request=req,
                                 content_type="text/plain")
         elif image['status'] == 'deleted':
-            msg = "Image %s not found." % id
-            LOG.debug(msg)
+            msg = _("Image %s not found.") % id
+            LOG.warn(msg)
             raise HTTPNotFound(explanation=msg, request=req,
                                content_type="text/plain")
 
@@ -1085,24 +1102,21 @@ class Controller(controller.BaseController):
         except exception.NotFound as e:
             msg = (_("Failed to find image to delete: %s") %
                    utils.exception_to_str(e))
-            for line in msg.split('\n'):
-                LOG.info(line)
+            LOG.warn(msg)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
             msg = (_("Forbidden to delete image: %s") %
                    utils.exception_to_str(e))
-            for line in msg.split('\n'):
-                LOG.info(line)
+            LOG.warn(msg)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
         except exception.InUseByStore as e:
-            msg = (_LI("Image %s could not be deleted because it is in use: "
-                       "%s") % (id, utils.exception_to_str(e)))
-            for line in msg.split('\n'):
-                LOG.info(line)
+            msg = (_("Image %(id)s could not be deleted because it is in use: "
+                     "%(exc)s") % {"id": id, "exc": utils.exception_to_str(e)})
+            LOG.warn(msg)
             raise HTTPConflict(explanation=msg,
                                request=req,
                                content_type="text/plain")
@@ -1123,8 +1137,8 @@ class Controller(controller.BaseController):
         try:
             return store.get_store_from_scheme(scheme)
         except store.UnknownScheme:
-            msg = "Store for scheme %s not found" % scheme
-            LOG.debug(msg)
+            msg = _("Store for scheme %s not found") % scheme
+            LOG.warn(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=request,
                                  content_type='text/plain')
