@@ -14,7 +14,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
 import webob.exc
 
 from glance.api import policy
@@ -22,10 +23,11 @@ from glance.api.v1 import controller
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
-import glance.openstack.common.log as logging
+from glance import i18n
 import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
 CONF = cfg.CONF
 CONF.import_opt('image_member_quota', 'glance.common.config')
 
@@ -72,12 +74,12 @@ class Controller(controller.BaseController):
         try:
             members = registry.get_image_members(req.context, image_id)
         except exception.NotFound:
-            msg = "Image with identifier %s not found" % image_id
-            LOG.debug(msg)
+            msg = _("Image with identifier %s not found") % image_id
+            LOG.warn(msg)
             raise webob.exc.HTTPNotFound(msg)
         except exception.Forbidden:
-            msg = "Unauthorized image access"
-            LOG.debug(msg)
+            msg = _("Unauthorized image access")
+            LOG.warn(msg)
             raise webob.exc.HTTPForbidden(msg)
         return dict(members=members)
 
