@@ -12,25 +12,23 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-import http.client as http
-
 import glance_store
 from oslo_log import log as logging
 import webob.exc
 
 from glance.api import policy
-from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
-from glance.i18n import _LI
+from glance import i18n
 import glance.notifier
 
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LI = i18n._LI
 
 
 class ImageActionsController(object):
@@ -45,60 +43,30 @@ class ImageActionsController(object):
 
     @utils.mutating
     def deactivate(self, req, image_id):
-        image_repo = self.gateway.get_repo(req.context,
-                                           authorization_layer=False)
+        image_repo = self.gateway.get_repo(req.context)
         try:
-            # FIXME(danms): This will still enforce the get_image policy
-            # which we don't want
             image = image_repo.get(image_id)
-
-            # NOTE(abhishekk): This is the right place to check whether user
-            # have permission to deactivate the image and remove the policy
-            # check later from the policy layer.
-            api_pol = api_policy.ImageAPIPolicy(req.context, image,
-                                                self.policy)
-            api_pol.deactivate_image()
-
-            status = image.status
             image.deactivate()
-            # not necessary to change the status if it's already 'deactivated'
-            if status == 'active':
-                image_repo.save(image, from_state='active')
-            LOG.info(_LI("Image %s is deactivated"), image_id)
+            image_repo.save(image)
+            LOG.info(_LI("Image %s is deactivated") % image_id)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to deactivate image '%s'", image_id)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.InvalidImageStatusTransition as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
 
     @utils.mutating
     def reactivate(self, req, image_id):
-        image_repo = self.gateway.get_repo(req.context,
-                                           authorization_layer=False)
+        image_repo = self.gateway.get_repo(req.context)
         try:
-            # FIXME(danms): This will still enforce the get_image policy
-            # which we don't want
             image = image_repo.get(image_id)
-
-            # NOTE(abhishekk): This is the right place to check whether user
-            # have permission to reactivate the image and remove the policy
-            # check later from the policy layer.
-            api_pol = api_policy.ImageAPIPolicy(req.context, image,
-                                                self.policy)
-            api_pol.reactivate_image()
-
-            status = image.status
             image.reactivate()
-            # not necessary to change the status if it's already 'active'
-            if status == 'deactivated':
-                image_repo.save(image, from_state='deactivated')
-            LOG.info(_LI("Image %s is reactivated"), image_id)
+            image_repo.save(image)
+            LOG.info(_LI("Image %s is reactivated") % image_id)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to reactivate image '%s'", image_id)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.InvalidImageStatusTransition as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
@@ -107,10 +75,10 @@ class ImageActionsController(object):
 class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def deactivate(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
     def reactivate(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
 
 def create_resource():
