@@ -16,7 +16,10 @@
 import re
 
 import glance_store
-from oslo.config import cfg
+from oslo.serialization import jsonutils as json
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import timeutils
 import six
 import six.moves.urllib.parse as urlparse
 import webob.exc
@@ -28,16 +31,13 @@ from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
+from glance import i18n
 import glance.notifier
-from glance.openstack.common import gettextutils
-from glance.openstack.common import jsonutils as json
-import glance.openstack.common.log as logging
-from glance.openstack.common import timeutils
 import glance.schema
 
 LOG = logging.getLogger(__name__)
-_LI = gettextutils._LI
-_LW = gettextutils._LW
+_ = i18n._
+_LW = i18n._LW
 
 CONF = cfg.CONF
 CONF.import_opt('disk_formats', 'glance.common.config', group='image_format')
@@ -65,21 +65,35 @@ class ImagesController(object):
             image_repo.add(image)
         except exception.DuplicateLocation as dup:
             raise webob.exc.HTTPBadRequest(explanation=dup.msg)
+        except exception.Invalid as e:
+            raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.Forbidden as e:
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.InvalidParameterValue as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.LimitExceeded as e:
-            LOG.info(utils.exception_to_str(e))
+            LOG.warn(utils.exception_to_str(e))
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=e.msg, request=req, content_type='text/plain')
         except exception.Duplicate as dupex:
             raise webob.exc.HTTPConflict(explanation=dupex.msg)
+        except exception.ReservedProperty as e:
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+        except exception.ReadonlyProperty as e:
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+        except TypeError as e:
+            LOG.debug(utils.exception_to_str(e))
+            raise webob.exc.HTTPBadRequest(
+                explanation=utils.exception_to_str(e))
 
         return image
 
-    def index(self, req, marker=None, limit=None, sort_key='created_at',
-              sort_dir='desc', filters=None, member_status='accepted'):
+    def index(self, req, marker=None, limit=None, sort_key=None,
+              sort_dir=None, filters=None, member_status='accepted'):
+        sort_key = ['created_at'] if not sort_key else sort_key
+
+        sort_dir = ['desc'] if not sort_dir else sort_dir
+
         result = {}
         if filters is None:
             filters = {}
@@ -92,7 +106,8 @@ class ImagesController(object):
         image_repo = self.gateway.get_repo(req.context)
         try:
             images = image_repo.list(marker=marker, limit=limit,
-                                     sort_key=sort_key, sort_dir=sort_dir,
+                                     sort_key=sort_key,
+                                     sort_dir=sort_dir,
                                      filters=filters,
                                      member_status=member_status)
             if len(images) != 0 and len(images) == limit:
@@ -130,18 +145,20 @@ class ImagesController(object):
                 image_repo.save(image)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except exception.Invalid as e:
+            raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.Forbidden as e:
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.InvalidParameterValue as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.StorageQuotaFull as e:
-            msg = (_LI("Denying attempt to upload image because it exceeds the"
-                       " .quota: %s") % utils.exception_to_str(e))
-            LOG.info(msg)
+            msg = (_("Denying attempt to upload image because it exceeds the"
+                     " quota: %s") % utils.exception_to_str(e))
+            LOG.warn(msg)
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=msg, request=req, content_type='text/plain')
         except exception.LimitExceeded as e:
-            LOG.info(utils.exception_to_str(e))
+            LOG.exception(utils.exception_to_str(e))
             raise webob.exc.HTTPRequestEntityTooLarge(
                 explanation=e.msg, request=req, content_type='text/plain')
 
@@ -166,14 +183,19 @@ class ImagesController(object):
         path = change['path']
         path_root = path[0]
         value = change['value']
+        json_schema_version = change.get('json_schema_version', 10)
         if path_root == 'locations':
             self._do_add_locations(image, path[1], value)
         else:
-            if (hasattr(image, path_root) or
-                    path_root in image.extra_properties):
+            if ((hasattr(image, path_root) or
+                    path_root in image.extra_properties)
+                    and json_schema_version == 4):
                 msg = _("Property %s already present.")
                 raise webob.exc.HTTPConflict(msg % path_root)
-            image.extra_properties[path_root] = value
+            if hasattr(image, path_root):
+                setattr(image, path_root, value)
+            else:
+                image.extra_properties[path_root] = value
 
     def _do_remove(self, req, image, change):
         path = change['path']
@@ -202,12 +224,14 @@ class ImagesController(object):
         except exception.NotFound as e:
             msg = (_("Failed to find image %(image_id)s to delete") %
                    {'image_id': image_id})
-            LOG.info(msg)
+            LOG.warn(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
         except exception.InUseByStore as e:
-            msg = (_LI("Image %s could not be deleted "
-                       "because it is in use: %s") % (image_id, e.msg))
-            LOG.info(msg)
+            msg = (_("Image %(id)s could not be deleted "
+                     "because it is in use: %(exc)s") %
+                   {"id": image_id,
+                    "exc": e.msg})
+            LOG.warn(msg)
             raise webob.exc.HTTPConflict(explanation=msg)
 
     def _get_locations_op_pos(self, path_pos, max_pos, allow_max):
@@ -218,7 +242,7 @@ class ImagesController(object):
             pos = int(path_pos)
         elif path_pos != '-':
             return None
-        if (not allow_max) and (pos not in range(max_pos)):
+        if not (allow_max or 0 <= pos < max_pos):
             return None
         return pos
 
@@ -241,8 +265,8 @@ class ImagesController(object):
             except (exception.BadStoreUri, exception.DuplicateLocation) as bse:
                 raise webob.exc.HTTPBadRequest(explanation=bse.msg)
             except ValueError as ve:    # update image status failed.
-                raise webob.exc.HTTPBadRequest(explanation=
-                                               utils.exception_to_str(ve))
+                raise webob.exc.HTTPBadRequest(
+                    explanation=utils.exception_to_str(ve))
 
     def _do_add_locations(self, image, path_pos, value):
         pos = self._get_locations_op_pos(path_pos,
@@ -257,8 +281,8 @@ class ImagesController(object):
         except (exception.BadStoreUri, exception.DuplicateLocation) as bse:
             raise webob.exc.HTTPBadRequest(explanation=bse.msg)
         except ValueError as ve:    # update image status failed.
-            raise webob.exc.HTTPBadRequest(explanation=
-                                           utils.exception_to_str(ve))
+            raise webob.exc.HTTPBadRequest(
+                explanation=utils.exception_to_str(ve))
 
     def _do_remove_locations(self, image, path_pos):
         pos = self._get_locations_op_pos(path_pos,
@@ -271,26 +295,37 @@ class ImagesController(object):
             # from the backend store.
             image.locations.pop(pos)
         except Exception as e:
-            raise webob.exc.HTTPInternalServerError(explanation=
-                                                    utils.exception_to_str(e))
-        if (len(image.locations) == 0) and (image.status == 'active'):
+            raise webob.exc.HTTPInternalServerError(
+                explanation=utils.exception_to_str(e))
+        if len(image.locations) == 0 and image.status == 'active':
             image.status = 'queued'
 
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
-    _disallowed_properties = ['direct_url', 'self', 'file', 'schema']
-    _readonly_properties = ['created_at', 'updated_at', 'status', 'checksum',
+    _disallowed_properties = ('direct_url', 'self', 'file', 'schema')
+    _readonly_properties = ('created_at', 'updated_at', 'status', 'checksum',
                             'size', 'virtual_size', 'direct_url', 'self',
-                            'file', 'schema']
-    _reserved_properties = ['owner', 'is_public', 'location', 'deleted',
-                            'deleted_at']
-    _base_properties = ['checksum', 'created_at', 'container_format',
+                            'file', 'schema')
+    _reserved_properties = ('owner', 'location', 'deleted', 'deleted_at')
+    _base_properties = ('checksum', 'created_at', 'container_format',
                         'disk_format', 'id', 'min_disk', 'min_ram', 'name',
                         'size', 'virtual_size', 'status', 'tags',
-                        'updated_at', 'visibility', 'protected']
+                        'updated_at', 'visibility', 'protected')
+    _available_sort_keys = ('name', 'status', 'container_format',
+                            'disk_format', 'size', 'id', 'created_at',
+                            'updated_at')
+
+    _default_sort_key = 'created_at'
+
+    _default_sort_dir = 'desc'
+
     _path_depth_limits = {'locations': {'add': 2, 'remove': 2, 'replace': 1}}
 
+    _default_sort_dir = 'desc'
+
+    _supported_operations = ('add', 'remove', 'replace')
+
     def __init__(self, schema=None):
         super(RequestDeserializer, self).__init__()
         self.schema = schema or get_schema()
@@ -307,8 +342,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         for key in cls._disallowed_properties:
             if key in image:
                 msg = _("Attribute '%s' is read-only.") % key
-                raise webob.exc.HTTPForbidden(explanation=
-                                              utils.exception_to_str(msg))
+                raise webob.exc.HTTPForbidden(
+                    explanation=utils.exception_to_str(msg))
 
     def create(self, request):
         body = self._get_request_body(request)
@@ -319,7 +354,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         image = {}
         properties = body
-        tags = properties.pop('tags', None)
+        tags = properties.pop('tags', [])
         for key in self._base_properties:
             try:
                 # NOTE(flwang): Instead of changing the _check_unexpected
@@ -334,15 +369,23 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         return dict(image=image, extra_properties=properties, tags=tags)
 
     def _get_change_operation_d10(self, raw_change):
-        try:
-            return raw_change['op']
-        except KeyError:
-            msg = _("Unable to find '%s' in JSON Schema change") % 'op'
+        op = raw_change.get('op')
+        if op is None:
+            msg = _('Unable to find `op` in JSON Schema change. '
+                    'It must be one of the following: %(available)s.') % \
+                {'available': ', '.join(self._supported_operations)}
             raise webob.exc.HTTPBadRequest(explanation=msg)
+        if op not in self._supported_operations:
+            msg = _('Invalid operation: `%(op)s`. '
+                    'It must be one of the following: %(available)s.') % \
+                {'op': op,
+                 'available': ', '.join(self._supported_operations)}
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        return op
 
     def _get_change_operation_d4(self, raw_change):
         op = None
-        for key in ['replace', 'add', 'remove']:
+        for key in self._supported_operations:
             if key in raw_change:
                 if op is not None:
                     msg = _('Operation objects must contain only one member'
@@ -492,7 +535,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
             # NOTE(zhiyan): the 'path' is a list.
             self._validate_path(op, path)
-            change = {'op': op, 'path': path}
+            change = {'op': op, 'path': path,
+                      'json_schema_version': json_schema_version}
 
             if not op == 'remove':
                 change['value'] = self._get_change_value(raw_change, op)
@@ -515,6 +559,16 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
         return limit
 
+    def _validate_sort_key(self, sort_key):
+        if sort_key not in self._available_sort_keys:
+            msg = _('Invalid sort key: %(sort_key)s. '
+                    'It must be one of the following: %(available)s.') % \
+                {'sort_key': sort_key,
+                 'available': ', '.join(self._available_sort_keys)}
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        return sort_key
+
     def _validate_sort_dir(self, sort_dir):
         if sort_dir not in ['asc', 'desc']:
             msg = _('Invalid sort direction: %s') % sort_dir
@@ -542,11 +596,67 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
         return filters
 
+    def _get_sorting_params(self, params):
+        """
+        Process sorting params.
+        Currently glance supports two sorting syntax: classic and new one,
+        that is uniform for all Openstack projects.
+        Classic syntax: sort_key=name&sort_dir=asc&sort_key=size&sort_dir=desc
+        New syntax: sort=name:asc,size:desc
+        """
+        sort_keys = []
+        sort_dirs = []
+
+        if 'sort' in params:
+            # use new sorting syntax here
+            if 'sort_key' in params or 'sort_dir' in params:
+                msg = _('Old and new sorting syntax cannot be combined')
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+            for sort_param in params.pop('sort').strip().split(','):
+                key, _sep, dir = sort_param.partition(':')
+                if not dir:
+                    dir = self._default_sort_dir
+                sort_keys.append(self._validate_sort_key(key.strip()))
+                sort_dirs.append(self._validate_sort_dir(dir.strip()))
+        else:
+            # continue with classic syntax
+            # NOTE(mfedosin): we have 3 options here:
+            # 1. sort_dir wasn't passed: we use default one - 'desc'.
+            # 2. Only one sort_dir was passed: use it for every sort_key
+            # in the list.
+            # 3. Multiple sort_dirs were passed: consistently apply each one to
+            # the corresponding sort_key.
+            # If number of sort_dirs and sort_keys doesn't match then raise an
+            # exception.
+            while 'sort_key' in params:
+                sort_keys.append(self._validate_sort_key(
+                    params.pop('sort_key').strip()))
+
+            while 'sort_dir' in params:
+                sort_dirs.append(self._validate_sort_dir(
+                    params.pop('sort_dir').strip()))
+
+            if sort_dirs:
+                dir_len = len(sort_dirs)
+                key_len = len(sort_keys)
+
+                if dir_len > 1 and dir_len != key_len:
+                    msg = _('Number of sort dirs does not match the number '
+                            'of sort keys')
+                    raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        if not sort_keys:
+            sort_keys = [self._default_sort_key]
+
+        if not sort_dirs:
+            sort_dirs = [self._default_sort_dir]
+
+        return sort_keys, sort_dirs
+
     def index(self, request):
         params = request.params.copy()
         limit = params.pop('limit', None)
         marker = params.pop('marker', None)
-        sort_dir = params.pop('sort_dir', 'desc')
         member_status = params.pop('member_status', 'accepted')
 
         # NOTE (flwang) To avoid using comma or any predefined chars to split
@@ -557,8 +667,6 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             tags.append(params.pop('tag').strip())
 
         query_params = {
-            'sort_key': params.pop('sort_key', 'created_at'),
-            'sort_dir': self._validate_sort_dir(sort_dir),
             'filters': self._get_filters(params),
             'member_status': self._validate_member_status(member_status),
         }
@@ -572,6 +680,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if tags:
             query_params['filters']['tags'] = tags
 
+        # NOTE(mfedosin): param is still called sort_key and sort_dir,
+        # instead of sort_keys and sort_dirs respectively.
+        # It's done because in v1 it's still a single value.
+
+        query_params['sort_key'], query_params['sort_dir'] = \
+            self._get_sorting_params(params)
+
         return query_params
 
 
@@ -684,7 +799,7 @@ def get_base_properties():
                         '-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$'),
         },
         'name': {
-            'type': 'string',
+            'type': ['null', 'string'],
             'description': _('Descriptive name for the image'),
             'maxLength': 255,
         },
@@ -704,46 +819,46 @@ def get_base_properties():
             'description': _('If true, image will not be deletable.'),
         },
         'checksum': {
-            'type': 'string',
+            'type': ['null', 'string'],
             'description': _('md5 hash of image contents. (READ-ONLY)'),
             'maxLength': 32,
         },
         'owner': {
-            'type': 'string',
+            'type': ['null', 'string'],
             'description': _('Owner of the image'),
             'maxLength': 255,
         },
         'size': {
-            'type': 'integer',
+            'type': ['null', 'integer'],
             'description': _('Size of image file in bytes (READ-ONLY)'),
         },
         'virtual_size': {
-            'type': 'integer',
+            'type': ['null', 'integer'],
             'description': _('Virtual size of image in bytes (READ-ONLY)'),
         },
         'container_format': {
-            'type': 'string',
+            'type': ['null', 'string'],
             'description': _('Format of the container'),
-            'enum': CONF.image_format.container_formats,
+            'enum': [None] + CONF.image_format.container_formats,
         },
         'disk_format': {
-            'type': 'string',
+            'type': ['null', 'string'],
             'description': _('Format of the disk'),
-            'enum': CONF.image_format.disk_formats,
+            'enum': [None] + CONF.image_format.disk_formats,
         },
         'created_at': {
             'type': 'string',
             'description': _('Date and time of image registration'
                              ' (READ-ONLY)'),
-            #TODO(bcwaldon): our jsonschema library doesn't seem to like the
+            # TODO(bcwaldon): our jsonschema library doesn't seem to like the
             # format attribute, figure out why!
-            #'format': 'date-time',
+            # 'format': 'date-time',
         },
         'updated_at': {
             'type': 'string',
             'description': _('Date and time of the last image modification'
                              ' (READ-ONLY)'),
-            #'format': 'date-time',
+            # 'format': 'date-time',
         },
         'tags': {
             'type': 'array',
