@@ -13,149 +13,92 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import http.client as http
-
+from oslo.serialization import jsonutils
+from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import strutils
+import six
 import webob.exc
 from wsme.rest import json
 
 from glance.api import policy
 from glance.api.v2.model.metadef_tag import MetadefTag
 from glance.api.v2.model.metadef_tag import MetadefTags
-from glance.api.v2 import policy as api_policy
 from glance.common import exception
+from glance.common import utils
 from glance.common import wsgi
 from glance.common import wsme_utils
 import glance.db
-from glance.i18n import _
+from glance import i18n
 import glance.notifier
 import glance.schema
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LE = i18n._LE
+_LI = i18n._LI
+
+CONF = cfg.CONF
 
 
 class TagsController(object):
-    def __init__(self, db_api=None, policy_enforcer=None, notifier=None,
-                 schema=None):
+    def __init__(self, db_api=None, policy_enforcer=None, notifier=None):
         self.db_api = db_api or glance.db.get_api()
         self.policy = policy_enforcer or policy.Enforcer()
         self.notifier = notifier or glance.notifier.Notifier()
         self.gateway = glance.gateway.Gateway(db_api=self.db_api,
                                               notifier=self.notifier,
                                               policy_enforcer=self.policy)
-        self.schema = schema or get_schema()
         self.tag_schema_link = '/v2/schemas/metadefs/tag'
 
     def create(self, req, namespace, tag_name):
         tag_factory = self.gateway.get_metadef_tag_factory(req.context)
         tag_repo = self.gateway.get_metadef_tag_repo(req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
-        try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
         tag_name_as_dict = {'name': tag_name}
         try:
-            self.schema.validate(tag_name_as_dict)
-        except exception.InvalidObject as e:
-            raise webob.exc.HTTPBadRequest(explanation=e.msg)
-        try:
-            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
-            # Here we are just checking if user is authorized to create metadef
-            # tag or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).add_metadef_tag()
-
             new_meta_tag = tag_factory.new_tag(
                 namespace=namespace,
                 **tag_name_as_dict)
             tag_repo.add(new_meta_tag)
-        except exception.Invalid as e:
-            msg = (_("Couldn't create metadata tag: %s") % e)
-            raise webob.exc.HTTPBadRequest(explanation=msg)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to create metadata tag within "
-                      "'%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         return MetadefTag.to_wsme_model(new_meta_tag)
 
     def create_tags(self, req, metadata_tags, namespace):
         tag_factory = self.gateway.get_metadef_tag_factory(req.context)
         tag_repo = self.gateway.get_metadef_tag_repo(req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
-            # Here we are just checking if user is authorized to create metadef
-            # tag or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).add_metadef_tags()
-
-            can_append = strutils.bool_from_string(req.headers.get(
-                'X-Openstack-Append'))
-
             tag_list = []
             for metadata_tag in metadata_tags.tags:
                 tag_list.append(tag_factory.new_tag(
                     namespace=namespace, **metadata_tag.to_dict()))
-            tag_repo.add_tags(tag_list, can_append)
+            tag_repo.add_tags(tag_list)
             tag_list_out = [MetadefTag(**{'name': db_metatag.name})
                             for db_metatag in tag_list]
             metadef_tags = MetadefTags()
             metadef_tags.tags = tag_list_out
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to create metadata tags within "
-                      "'%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         return metadef_tags
 
     def index(self, req, namespace, marker=None, limit=None,
               sort_key='created_at', sort_dir='desc', filters=None):
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): This is just a "do you have permission to
-            # list tags" check. Each object is checked against
-            # get_metadef_tag below.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).get_metadef_tags()
-
             filters = filters or dict()
             filters['namespace'] = namespace
 
@@ -174,111 +117,61 @@ class TagsController(object):
             metadef_tags = MetadefTags()
             metadef_tags.tags = tag_list
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to retrieve metadata tags "
-                      "within '%s' namespace", namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         return metadef_tags
 
     def show(self, req, namespace, tag_name):
         meta_tag_repo = self.gateway.get_metadef_tag_repo(req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef tags are associated with
-            # namespace, so made provision to pass namespace here
-            # for visibility check
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).get_metadef_tag()
-
             metadef_tag = meta_tag_repo.get(namespace, tag_name)
             return MetadefTag.to_wsme_model(metadef_tag)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to show metadata tag '%s' "
-                      "within '%s' namespace", tag_name, namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
     def update(self, req, metadata_tag, namespace, tag_name):
         meta_repo = self.gateway.get_metadef_tag_repo(req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
-            # Here we are just checking if user is authorized to update metadef
-            # tag or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).modify_metadef_tag()
-
             metadef_tag = meta_repo.get(namespace, tag_name)
             metadef_tag._old_name = metadef_tag.name
             metadef_tag.name = wsme_utils._get_value(
                 metadata_tag.name)
             updated_metadata_tag = meta_repo.save(metadef_tag)
-        except exception.Invalid as e:
-            msg = (_("Couldn't update metadata tag: %s") % e)
-            raise webob.exc.HTTPBadRequest(explanation=msg)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to update metadata tag '%s' "
-                      "within '%s' namespace", tag_name, namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Duplicate as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
         return MetadefTag.to_wsme_model(updated_metadata_tag)
 
     def delete(self, req, namespace, tag_name):
         meta_repo = self.gateway.get_metadef_tag_repo(req.context)
-        ns_repo = self.gateway.get_metadef_namespace_repo(req.context)
         try:
-            namespace_obj = ns_repo.get(namespace)
-        except (exception.Forbidden, exception.NotFound):
-            # NOTE (abhishekk): Returning 404 Not Found as the
-            # namespace is outside of this user's project
-            msg = _("Namespace %s not found") % namespace
-            raise webob.exc.HTTPNotFound(explanation=msg)
-
-        try:
-            # NOTE(abhishekk): Metadef tags is created for Metadef namespaces
-            # Here we are just checking if user is authorized to delete metadef
-            # tag or not.
-            api_policy.MetadefAPIPolicy(
-                req.context,
-                md_resource=namespace_obj,
-                enforcer=self.policy).delete_metadef_tag()
-
             metadef_tag = meta_repo.get(namespace, tag_name)
             metadef_tag.delete()
             meta_repo.remove(metadef_tag)
         except exception.Forbidden as e:
-            LOG.debug("User not permitted to delete metadata tag '%s' "
-                      "within '%s' namespace", tag_name, namespace)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except Exception as e:
+            LOG.error(utils.exception_to_str(e))
+            raise webob.exc.HTTPInternalServerError()
 
 
 def _get_base_definitions():
@@ -288,19 +181,18 @@ def _get_base_definitions():
 def _get_base_properties():
     return {
         "name": {
-            "type": "string",
-            "maxLength": 80
+            "type": "string"
         },
         "created_at": {
             "type": "string",
-            "readOnly": True,
-            "description": _("Date and time of tag creation"),
+            "description": _("Date and time of tag creation"
+                             " (READ-ONLY)"),
             "format": "date-time"
         },
         "updated_at": {
             "type": "string",
-            "readOnly": True,
-            "description": _("Date and time of the last tag modification"),
+            "description": _("Date and time of the last tag modification"
+                             " (READ-ONLY)"),
             "format": "date-time"
         }
     }
@@ -452,34 +344,34 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         self.schema = schema or get_schema()
 
     def create(self, response, metadata_tag):
-        response.status_int = http.CREATED
+        response.status_int = 201
         self.show(response, metadata_tag)
 
     def create_tags(self, response, result):
-        response.status_int = http.CREATED
+        response.status_int = 201
         metadata_tags_json = json.tojson(MetadefTags, result)
         body = jsonutils.dumps(metadata_tags_json, ensure_ascii=False)
-        response.unicode_body = body
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def show(self, response, metadata_tag):
         metadata_tag_json = json.tojson(MetadefTag, metadata_tag)
         body = jsonutils.dumps(metadata_tag_json, ensure_ascii=False)
-        response.unicode_body = body
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def update(self, response, metadata_tag):
-        response.status_int = http.OK
+        response.status_int = 200
         self.show(response, metadata_tag)
 
     def index(self, response, result):
         metadata_tags_json = json.tojson(MetadefTags, result)
         body = jsonutils.dumps(metadata_tags_json, ensure_ascii=False)
-        response.unicode_body = body
+        response.unicode_body = six.text_type(body)
         response.content_type = 'application/json'
 
     def delete(self, response, result):
-        response.status_int = http.NO_CONTENT
+        response.status_int = 204
 
 
 def get_tag_href(namespace_name, metadef_tag):
