@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from glance.api.v2 import image_actions
 from glance.api.v2 import image_data
 from glance.api.v2 import image_members
 from glance.api.v2 import image_tags
@@ -21,6 +22,7 @@ from glance.api.v2 import metadef_namespaces
 from glance.api.v2 import metadef_objects
 from glance.api.v2 import metadef_properties
 from glance.api.v2 import metadef_resource_types
+from glance.api.v2 import metadef_tags
 from glance.api.v2 import schemas
 from glance.api.v2 import tasks
 from glance.common import wsgi
@@ -186,6 +188,28 @@ class API(wsgi.Router):
                        conditions={'method': ['POST', 'PUT', 'DELETE',
                                               'PATCH', 'HEAD']})
 
+        mapper.connect('/schemas/metadefs/tag',
+                       controller=schemas_resource,
+                       action='metadef_tag',
+                       conditions={'method': ['GET']})
+        mapper.connect('/schemas/metadefs/tag',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET',
+                       conditions={'method': ['POST', 'PUT', 'DELETE',
+                                              'PATCH', 'HEAD']})
+
+        mapper.connect('/schemas/metadefs/tags',
+                       controller=schemas_resource,
+                       action='metadef_tags',
+                       conditions={'method': ['GET']})
+        mapper.connect('/schemas/metadefs/tags',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET',
+                       conditions={'method': ['POST', 'PUT', 'DELETE',
+                                              'PATCH', 'HEAD']})
+
         # Metadef resource types
         metadef_resource_types_resource = (
             metadef_resource_types.create_resource())
@@ -348,6 +372,48 @@ class API(wsgi.Router):
                        allowed_methods='GET, PUT, DELETE',
                        conditions={'method': ['POST', 'PATCH', 'HEAD']})
 
+        # Metadef tags
+        metadef_tags_resource = metadef_tags.create_resource()
+        mapper.connect('/metadefs/namespaces/{namespace}/tags',
+                       controller=metadef_tags_resource,
+                       action='index',
+                       conditions={'method': ['GET']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags',
+                       controller=metadef_tags_resource,
+                       action='create_tags',
+                       conditions={'method': ['POST']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags',
+                       controller=metadef_namespace_resource,
+                       action='delete_tags',
+                       conditions={'method': ['DELETE']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET, POST, DELETE',
+                       conditions={'method': ['PUT', 'PATCH', 'HEAD']})
+
+        mapper.connect('/metadefs/namespaces/{namespace}/tags/{tag_name}',
+                       controller=metadef_tags_resource,
+                       action='show',
+                       conditions={'method': ['GET']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags/{tag_name}',
+                       controller=metadef_tags_resource,
+                       action='create',
+                       conditions={'method': ['POST']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags/{tag_name}',
+                       controller=metadef_tags_resource,
+                       action='update',
+                       conditions={'method': ['PUT']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags/{tag_name}',
+                       controller=metadef_tags_resource,
+                       action='delete',
+                       conditions={'method': ['DELETE']})
+        mapper.connect('/metadefs/namespaces/{namespace}/tags/{tag_name}',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='GET, POST, PUT, DELETE',
+                       conditions={'method': ['PATCH', 'HEAD']})
+
         images_resource = images.create_resource(custom_image_properties)
         mapper.connect('/images',
                        controller=images_resource,
@@ -382,6 +448,28 @@ class API(wsgi.Router):
                        allowed_methods='GET, PATCH, DELETE',
                        conditions={'method': ['POST', 'PUT', 'HEAD']})
 
+        image_actions_resource = image_actions.create_resource()
+        mapper.connect('/images/{image_id}/actions/deactivate',
+                       controller=image_actions_resource,
+                       action='deactivate',
+                       conditions={'method': ['POST']})
+        mapper.connect('/images/{image_id}/actions/reactivate',
+                       controller=image_actions_resource,
+                       action='reactivate',
+                       conditions={'method': ['POST']})
+        mapper.connect('/images/{image_id}/actions/deactivate',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='POST',
+                       conditions={'method': ['GET', 'PUT', 'DELETE', 'PATCH',
+                                              'HEAD']})
+        mapper.connect('/images/{image_id}/actions/reactivate',
+                       controller=reject_method_resource,
+                       action='reject',
+                       allowed_methods='POST',
+                       conditions={'method': ['GET', 'PUT', 'DELETE', 'PATCH',
+                                              'HEAD']})
+
         image_data_resource = image_data.create_resource()
         mapper.connect('/images/{image_id}/file',
                        controller=image_data_resource,
