@@ -12,30 +12,37 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from collections import OrderedDict
 import ConfigParser
 import re
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_policy import policy
 
 import glance.api.policy
 from glance.common import exception
-from glance.openstack.common import log as logging
-from glance.openstack.common import policy
+from glance import i18n
 
 # NOTE(bourke): The default dict_type is collections.OrderedDict in py27, but
 # we must set manually for compatibility with py26
 CONFIG = ConfigParser.SafeConfigParser(dict_type=OrderedDict)
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LE = i18n._LE
 
 property_opts = [
     cfg.StrOpt('property_protection_file',
-               help=_('The location of the property protection file.')),
+               help=_('The location of the property protection file.'
+                      'This file contains the rules for property protections '
+                      'and the roles/policies associated with it. If this '
+                      'config value is not specified, by default, property '
+                      'protections won\'t be enforced. If a value is '
+                      'specified and the file is not found, then the '
+                      'glance-api service will not start.')),
     cfg.StrOpt('property_protection_rule_format',
                default='roles',
+               choices=('roles', 'policies'),
                help=_('This config value indicates whether "roles" or '
                       '"policies" are used in the property protection file.')),
 ]
@@ -71,17 +78,17 @@ class PropertyRules(object):
             conf_file = CONF.find_file(CONF.property_protection_file)
             CONFIG.read(conf_file)
         except Exception as e:
-            msg = (_("Couldn't find property protection file %(file)s: "
-                     "%(error)s.") % {'file': CONF.property_protection_file,
-                                      'error': e})
+            msg = (_LE("Couldn't find property protection file %(file)s: "
+                       "%(error)s.") % {'file': CONF.property_protection_file,
+                                        'error': e})
             LOG.error(msg)
             raise InvalidPropProtectConf()
 
         if self.prop_prot_rule_format not in ['policies', 'roles']:
-            msg = _("Invalid value '%s' for "
-                    "'property_protection_rule_format'. "
-                    "The permitted values are "
-                    "'roles' and 'policies'") % self.prop_prot_rule_format
+            msg = _LE("Invalid value '%s' for "
+                      "'property_protection_rule_format'. "
+                      "The permitted values are "
+                      "'roles' and 'policies'") % self.prop_prot_rule_format
             LOG.error(msg)
             raise InvalidPropProtectConf()
 
@@ -97,9 +104,9 @@ class PropertyRules(object):
                     if self.prop_prot_rule_format == 'policies':
                         if ',' in permissions:
                             LOG.error(
-                                _("Multiple policies '%s' not allowed"
-                                  "for a given operation. Policies can be "
-                                  "combined in the policy file"),
+                                _LE("Multiple policies '%s' not allowed "
+                                    "for a given operation. Policies can be "
+                                    "combined in the policy file"),
                                 permissions)
                             raise InvalidPropProtectConf()
                         self.prop_exp_mapping[compiled_rule] = property_exp
@@ -110,7 +117,7 @@ class PropertyRules(object):
                         permissions = [permission.strip() for permission in
                                        permissions.split(',')]
                         if '@' in permissions and '!' in permissions:
-                            msg = (_(
+                            msg = (_LE(
                                 "Malformed property protection rule in "
                                 "[%(prop)s] %(op)s=%(perm)s: '@' and '!' "
                                 "are mutually exclusive") %
@@ -135,9 +142,9 @@ class PropertyRules(object):
         try:
             return re.compile(rule)
         except Exception as e:
-            msg = (_("Encountered a malformed property protection rule"
-                     " %(rule)s: %(error)s.") % {'rule': rule,
-                                                 'error': e})
+            msg = (_LE("Encountered a malformed property protection rule"
+                       " %(rule)s: %(error)s.") % {'rule': rule,
+                                                   'error': e})
             LOG.error(msg)
             raise InvalidPropProtectConf()
 
@@ -150,18 +157,19 @@ class PropertyRules(object):
         then the corresponding policy rule would be:
         "prop_a:create": "rule:glance_creator"
         where glance_creator is defined in policy.json. For example:
-        "glance:creator": "role:admin or role:glance_create_user"
+        "glance_creator": "role:admin or role:glance_create_user"
         """
         rule = "rule:%s" % rule
         rule_name = "%s:%s" % (property_exp, action)
-        rule_dict = {}
-        rule_dict[rule_name] = policy.parse_rule(rule)
+        rule_dict = policy.Rules.from_dict({
+            rule_name: rule
+        })
         self.policy_enforcer.add_rules(rule_dict)
 
     def _check_policy(self, property_exp, action, context):
         try:
-            target = ":".join([property_exp, action])
-            self.policy_enforcer.enforce(context, target, {})
+            action = ":".join([property_exp, action])
+            self.policy_enforcer.enforce(context, action, {})
         except exception.Forbidden:
             return False
         return True
