@@ -19,7 +19,10 @@ RPC Controller
 import datetime
 import traceback
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+import oslo_utils.importutils as imp
+from oslo_utils import timeutils
 import six
 from webob import exc
 
@@ -27,11 +30,11 @@ from glance.common import client
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
-import glance.openstack.common.importutils as imp
-import glance.openstack.common.log as logging
-from glance.openstack.common import timeutils
+from glance import i18n
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LE = i18n._LE
 
 
 rpc_opts = [
@@ -43,7 +46,7 @@ rpc_opts = [
                          'exceptions',
                          ],
                 help='Modules of exceptions that are permitted to be recreated'
-                     'upon receiving exception data from an rpc call.'),
+                     ' upon receiving exception data from an rpc call.'),
 ]
 
 CONF = cfg.CONF
@@ -103,7 +106,7 @@ class Controller(object):
         Exports methods through the RPC Api.
 
         :params resource: Resource's instance to register.
-        :params filtered: List of methods that *can* me registered. Read
+        :params filtered: List of methods that *can* be registered. Read
         as "Method must be in this list".
         :params excluded: List of methods to exclude.
         :params refiner: Callable to use as filter for methods.
@@ -180,7 +183,7 @@ class Controller(object):
                     raise
 
                 cls, val = e.__class__, utils.exception_to_str(e)
-                msg = (_("RPC Call Error: %(val)s\n%(tb)s") %
+                msg = (_LE("RPC Call Error: %(val)s\n%(tb)s") %
                        dict(val=val, tb=traceback.format_exc()))
                 LOG.error(msg)
 
