@@ -17,6 +17,9 @@ __all__ = [
     'run',
 ]
 
+from oslo_concurrency import lockutils
+from oslo_log import log as logging
+from oslo_utils import excutils
 import six
 
 from glance.api.v2 import images as v2_api
@@ -25,15 +28,13 @@ from glance.common.scripts import utils as script_utils
 from glance.common import store_utils
 from glance.common import utils as common_utils
 from glance import i18n
-from glance.openstack.common import excutils
-from glance.openstack.common import lockutils
-import glance.openstack.common.log as logging
 
 
+LOG = logging.getLogger(__name__)
+_ = i18n._
 _LE = i18n._LE
 _LI = i18n._LI
 _LW = i18n._LW
-LOG = logging.getLogger(__name__)
 
 
 def run(t_id, context, task_repo, image_repo, image_factory):
@@ -67,14 +68,14 @@ def _execute(t_id, task_repo, image_repo, image_factory):
         # Note: The message string contains Error in it to indicate
         # in the task.message that it's a error message for the user.
 
-        #TODO(nikhil): need to bring back save_and_reraise_exception when
+        # TODO(nikhil): need to bring back save_and_reraise_exception when
         # necessary
         err_msg = ("Error: " + six.text_type(type(e)) + ': ' +
                    common_utils.exception_to_str(e))
-        log_msg = _LE(err_msg + ("Task ID %s" % task.task_id))
+        log_msg = _LE(err_msg + ("Task ID %s" % task.task_id))  # noqa
         LOG.exception(log_msg)
 
-        task.fail(_LE(err_msg))
+        task.fail(_LE(err_msg))  # noqa
     finally:
         task_repo.save(task)
 
@@ -131,9 +132,9 @@ def create_image(image_repo, image_factory, image_properties, task_id):
         try:
             properties[key] = image_properties.pop(key)
         except KeyError:
-            msg = _("Task ID %(task_id)s: Ignoring property %(k)s for setting "
-                    "base properties while creating "
-                    "Image.") % {'task_id': task_id, 'k': key}
+            msg = ("Task ID %(task_id)s: Ignoring property %(k)s for setting "
+                   "base properties while creating "
+                   "Image.") % {'task_id': task_id, 'k': key}
             LOG.debug(msg)
 
     # NOTE: get the rest of the properties and pass them as
