@@ -16,16 +16,21 @@
 
 import copy
 import functools
+from operator import itemgetter
 import uuid
 
+from oslo_log import log as logging
+from oslo_utils import timeutils
 import six
 
 from glance.common import exception
-import glance.openstack.common.log as logging
-from glance.openstack.common import timeutils
-
+from glance.common import utils
+from glance import i18n
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LI = i18n._LI
+_LW = i18n._LW
 
 DATA = {
     'images': {},
@@ -35,10 +40,17 @@ DATA = {
     'metadef_objects': [],
     'metadef_properties': [],
     'metadef_resource_types': [],
+    'metadef_tags': [],
     'tags': {},
     'locations': [],
     'tasks': {},
-    'task_info': {}
+    'task_info': {},
+    'artifacts': {},
+    'artifact_properties': {},
+    'artifact_tags': {},
+    'artifact_dependencies': {},
+    'artifact_blobs': {},
+    'artifact_blob_locations': {}
 }
 
 INDEX = 0
@@ -47,12 +59,13 @@ INDEX = 0
 def log_call(func):
     @functools.wraps(func)
     def wrapped(*args, **kwargs):
-        LOG.info(_('Calling %(funcname)s: args=%(args)s, kwargs=%(kwargs)s') %
+        LOG.info(_LI('Calling %(funcname)s: args=%(args)s, '
+                     'kwargs=%(kwargs)s') %
                  {"funcname": func.__name__,
                   "args": args,
                   "kwargs": kwargs})
         output = func(*args, **kwargs)
-        LOG.info(_('Returning %(funcname)s: %(output)s') %
+        LOG.info(_LI('Returning %(funcname)s: %(output)s') %
                  {"funcname": func.__name__,
                   "output": output})
         return output
@@ -69,10 +82,12 @@ def reset():
         'metadef_objects': [],
         'metadef_properties': [],
         'metadef_resource_types': [],
+        'metadef_tags': [],
         'tags': {},
         'locations': [],
         'tasks': {},
-        'task_info': {}
+        'task_info': {},
+        'artifacts': {}
     }
 
 
@@ -89,6 +104,7 @@ def _get_session():
     return DATA
 
 
+@utils.no_4byte_params
 def _image_location_format(image_id, value, meta_data, status, deleted=False):
     dt = timeutils.utcnow()
     return {
@@ -174,6 +190,20 @@ def _task_info_format(task_id, **values):
     return task_info
 
 
+@utils.no_4byte_params
+def _image_update(image, values, properties):
+    # NOTE(bcwaldon): store properties as a list to match sqlalchemy driver
+    properties = [{'name': k,
+                   'value': v,
+                   'image_id': image['id'],
+                   'deleted': False} for k, v in properties.items()]
+    if 'properties' not in image.keys():
+        image['properties'] = []
+    image['properties'].extend(properties)
+    image.update(values)
+    return image
+
+
 def _image_format(image_id, **values):
     dt = timeutils.utcnow()
     image = {
@@ -209,16 +239,7 @@ def _image_format(image_id, **values):
             image['locations'].append(location_ref)
             DATA['locations'].append(location_ref)
 
-    #NOTE(bcwaldon): store properties as a list to match sqlalchemy driver
-    properties = values.pop('properties', {})
-    properties = [{'name': k,
-                   'value': v,
-                   'image_id': image_id,
-                   'deleted': False} for k, v in properties.items()]
-    image['properties'] = properties
-
-    image.update(values)
-    return image
+    return _image_update(image, values, values.pop('properties', {}))
 
 
 def _filter_images(images, filters, context,
@@ -326,13 +347,35 @@ def _do_pagination(context, images, marker, limit, show_deleted,
 
 
 def _sort_images(images, sort_key, sort_dir):
-    reverse = False
-    if images and not (sort_key in images[0]):
-        raise exception.InvalidSortKey()
-    keyfn = lambda x: (x[sort_key] if x[sort_key] is not None else '',
-                       x['created_at'], x['id'])
-    reverse = sort_dir == 'desc'
-    images.sort(key=keyfn, reverse=reverse)
+    sort_key = ['created_at'] if not sort_key else sort_key
+
+    default_sort_dir = 'desc'
+
+    if not sort_dir:
+        sort_dir = [default_sort_dir] * len(sort_key)
+    elif len(sort_dir) == 1:
+        default_sort_dir = sort_dir[0]
+        sort_dir *= len(sort_key)
+
+    for key in ['created_at', 'id']:
+        if key not in sort_key:
+            sort_key.append(key)
+            sort_dir.append(default_sort_dir)
+
+    for key in sort_key:
+        if images and not (key in images[0]):
+            raise exception.InvalidSortKey()
+
+    if any(dir for dir in sort_dir if dir not in ['asc', 'desc']):
+        raise exception.InvalidSortDir()
+
+    if len(sort_key) != len(sort_dir):
+        raise exception.Invalid(message='Number of sort dirs does not match '
+                                        'the number of sort keys')
+
+    for key, dir in reversed(zip(sort_key, sort_dir)):
+        reverse = dir == 'desc'
+        images.sort(key=itemgetter(key), reverse=reverse)
 
     return images
 
@@ -341,15 +384,16 @@ def _image_get(context, image_id, force_show_deleted=False, status=None):
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        LOG.info(_('Could not find image %s') % image_id)
+        LOG.warn(_LW('Could not find image %s') % image_id)
         raise exception.NotFound()
 
-    if image['deleted'] and not (force_show_deleted or context.show_deleted):
-        LOG.info(_('Unable to get deleted image'))
+    if image['deleted'] and not (force_show_deleted
+                                 or context.can_see_deleted):
+        LOG.warn(_LW('Unable to get deleted image'))
         raise exception.NotFound()
 
     if not is_image_visible(context, image):
-        LOG.info(_('Unable to get unowned image'))
+        LOG.warn(_LW('Unable to get unowned image'))
         raise exception.Forbidden("Image not visible to you")
 
     return image
@@ -358,13 +402,13 @@ def _image_get(context, image_id, force_show_deleted=False, status=None):
 @log_call
 def image_get(context, image_id, session=None, force_show_deleted=False):
     image = _image_get(context, image_id, force_show_deleted)
-    return _normalize_locations(copy.deepcopy(image),
+    return _normalize_locations(context, copy.deepcopy(image),
                                 force_show_deleted=force_show_deleted)
 
 
 @log_call
 def image_get_all(context, filters=None, marker=None, limit=None,
-                  sort_key='created_at', sort_dir='desc',
+                  sort_key=None, sort_dir=None,
                   member_status='accepted', is_public=None,
                   admin_as_user=False, return_tag=False):
     filters = filters or {}
@@ -378,7 +422,7 @@ def image_get_all(context, filters=None, marker=None, limit=None,
     force_show_deleted = True if filters.get('deleted') else False
     res = []
     for image in images:
-        img = _normalize_locations(copy.deepcopy(image),
+        img = _normalize_locations(context, copy.deepcopy(image),
                                    force_show_deleted=force_show_deleted)
         if return_tag:
             img['tags'] = image_tag_get_all(context, img['id'])
@@ -463,7 +507,7 @@ def image_member_create(context, values):
 def image_member_update(context, member_id, values):
     global DATA
     for member in DATA['members']:
-        if (member['id'] == member_id):
+        if member['id'] == member_id:
             member.update(values)
             member['updated_at'] = timeutils.utcnow()
             return copy.deepcopy(member)
@@ -475,7 +519,7 @@ def image_member_update(context, member_id, values):
 def image_member_delete(context, member_id):
     global DATA
     for i, member in enumerate(DATA['members']):
-        if (member['id'] == member_id):
+        if member['id'] == member_id:
             del DATA['members'][i]
             break
     else:
@@ -483,6 +527,7 @@ def image_member_delete(context, member_id):
 
 
 @log_call
+@utils.no_4byte_params
 def image_location_add(context, image_id, location):
     deleted = location['status'] in ('deleted', 'pending_delete')
     location_ref = _image_location_format(image_id,
@@ -496,6 +541,7 @@ def image_location_add(context, image_id, location):
 
 
 @log_call
+@utils.no_4byte_params
 def image_location_update(context, image_id, location):
     loc_id = location.get('id')
     if loc_id is None:
@@ -508,7 +554,7 @@ def image_location_update(context, image_id, location):
 
     updated = False
     for loc in DATA['locations']:
-        if (loc['id'] == loc_id and loc['image_id'] == image_id):
+        if loc['id'] == loc_id and loc['image_id'] == image_id:
             loc.update({"value": location['url'],
                         "meta_data": location['metadata'],
                         "status": location['status'],
@@ -535,7 +581,7 @@ def image_location_delete(context, image_id, location_id, status,
 
     deleted = False
     for loc in DATA['locations']:
-        if (loc['id'] == location_id and loc['image_id'] == image_id):
+        if loc['id'] == location_id and loc['image_id'] == image_id:
             deleted = True
             delete_time = delete_time or timeutils.utcnow()
             loc.update({"deleted": deleted,
@@ -583,7 +629,7 @@ def _image_locations_delete_all(context, image_id, delete_time=None):
             del DATA['locations'][i]
 
 
-def _normalize_locations(image, force_show_deleted=False):
+def _normalize_locations(context, image, force_show_deleted=False):
     """
     Generate suitable dictionary list for locations field of image.
 
@@ -591,6 +637,11 @@ def _normalize_locations(image, force_show_deleted=False):
     from image query.
     """
 
+    if image['status'] == 'deactivated' and not context.is_admin:
+        # Locations are not returned for a deactivated image for non-admin user
+        image['locations'] = []
+        return image
+
     if force_show_deleted:
         locations = image['locations']
     else:
@@ -629,7 +680,7 @@ def image_create(context, image_values):
     DATA['images'][image_id] = image
     DATA['tags'][image_id] = image.pop('tags', [])
 
-    return _normalize_locations(copy.deepcopy(image))
+    return _normalize_locations(context, copy.deepcopy(image))
 
 
 @log_call
@@ -654,15 +705,10 @@ def image_update(context, image_id, image_values, purge_props=False,
             # this matches weirdness in the sqlalchemy api
             prop['deleted'] = True
 
-    # add in any completely new properties
-    image['properties'].extend([{'name': k, 'value': v,
-                                 'image_id': image_id, 'deleted': False}
-                                for k, v in new_properties.items()])
-
     image['updated_at'] = timeutils.utcnow()
-    image.update(image_values)
+    _image_update(image, image_values, new_properties)
     DATA['images'][image_id] = image
-    return _normalize_locations(copy.deepcopy(image))
+    return _normalize_locations(context, copy.deepcopy(image))
 
 
 @log_call
@@ -693,7 +739,8 @@ def image_destroy(context, image_id):
         for tag in tags:
             image_tag_delete(context, image_id, tag)
 
-        return _normalize_locations(copy.deepcopy(DATA['images'][image_id]))
+        return _normalize_locations(context,
+                                    copy.deepcopy(DATA['images'][image_id]))
     except KeyError:
         raise exception.NotFound()
 
@@ -719,6 +766,7 @@ def image_tag_set_all(context, image_id, values):
 
 
 @log_call
+@utils.no_4byte_params
 def image_tag_create(context, image_id, value):
     global DATA
     DATA['tags'][image_id].append(value)
@@ -858,18 +906,19 @@ def _task_get(context, task_id, force_show_deleted=False):
     try:
         task = DATA['tasks'][task_id]
     except KeyError:
-        msg = _('Could not find task %s') % task_id
-        LOG.info(msg)
+        msg = _LW('Could not find task %s') % task_id
+        LOG.warn(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
-    if task['deleted'] and not (force_show_deleted or context.show_deleted):
-        msg = _('Unable to get deleted task %s') % task_id
-        LOG.info(msg)
+    if task['deleted'] and not (force_show_deleted or context.can_see_deleted):
+        msg = _LW('Unable to get deleted task %s') % task_id
+        LOG.warn(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     if not _is_task_visible(context, task):
         msg = "Forbidding request, task %s is not visible" % task_id
         LOG.debug(msg)
+        msg = _("Forbidding request, task %s is not visible") % task_id
         raise exception.Forbidden(msg)
 
     task_info = _task_info_get(task_id)
@@ -965,7 +1014,6 @@ def _sort_tasks(tasks, sort_key, sort_dir):
                        x['created_at'], x['id'])
     reverse = sort_dir == 'desc'
     tasks.sort(key=keyfn, reverse=reverse)
-
     return tasks
 
 
@@ -1024,13 +1072,22 @@ def _task_info_get(task_id):
     try:
         task_info = DATA['task_info'][task_id]
     except KeyError:
-        msg = _('Could not find task info %s') % task_id
-        LOG.info(msg)
+        msg = _LW('Could not find task info %s') % task_id
+        LOG.warn(msg)
         raise exception.TaskNotFound(task_id=task_id)
 
     return task_info
 
 
+def _metadef_delete_namespace_content(get_func, key, context, namespace_name):
+    global DATA
+    metadefs = get_func(context, namespace_name)
+    data = DATA[key]
+    for metadef in metadefs:
+        data.remove(metadef)
+    return metadefs
+
+
 @log_call
 def metadef_namespace_create(context, values):
     """Create a namespace object"""
@@ -1345,6 +1402,13 @@ def metadef_object_delete(context, namespace_name, object_name):
     return object
 
 
+def metadef_object_delete_namespace_content(context, namespace_name,
+                                            session=None):
+    """Delete an object or raise if namespace or object doesn't exist."""
+    return _metadef_delete_namespace_content(
+        metadef_object_get_all, 'metadef_objects', context, namespace_name)
+
+
 @log_call
 def metadef_object_count(context, namespace_name):
     """Get metadef object count in a namespace"""
@@ -1521,6 +1585,14 @@ def metadef_property_delete(context, namespace_name, property_name):
     return property
 
 
+def metadef_property_delete_namespace_content(context, namespace_name,
+                                              session=None):
+    """Delete a property or raise if it or namespace doesn't exist."""
+    return _metadef_delete_namespace_content(
+        metadef_property_get_all, 'metadef_properties', context,
+        namespace_name)
+
+
 @log_call
 def metadef_resource_type_create(context, values):
     """Create a metadef resource type"""
@@ -1658,6 +1730,294 @@ def metadef_resource_type_association_delete(context, namespace_name,
     return resource_type
 
 
+@log_call
+def metadef_tag_get(context, namespace_name, name):
+    """Get a metadef tag"""
+    namespace = metadef_namespace_get(context, namespace_name)
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    for tag in DATA['metadef_tags']:
+        if tag['namespace_id'] == namespace['id'] and tag['name'] == name:
+            return tag
+    else:
+        msg = ("The metadata definition tag with name=%(name)s"
+               " was not found in namespace=%(namespace_name)s."
+               % {'name': name, 'namespace_name': namespace_name})
+        LOG.debug(msg)
+        raise exception.MetadefTagNotFound(name=name,
+                                           namespace_name=namespace_name)
+
+
+@log_call
+def metadef_tag_get_by_id(context, namespace_name, id):
+    """Get a metadef tag"""
+    namespace = metadef_namespace_get(context, namespace_name)
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    for tag in DATA['metadef_tags']:
+        if tag['namespace_id'] == namespace['id'] and tag['id'] == id:
+            return tag
+    else:
+        msg = (_("Metadata definition tag not found for id=%s") % id)
+        LOG.warn(msg)
+        raise exception.MetadefTagNotFound(msg)
+
+
+@log_call
+def metadef_tag_get_all(context, namespace_name, filters=None, marker=None,
+                        limit=None, sort_key='created_at', sort_dir=None,
+                        session=None):
+    """Get a metadef tags list"""
+
+    namespace = metadef_namespace_get(context, namespace_name)
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    tags = []
+    for tag in DATA['metadef_tags']:
+        if tag['namespace_id'] == namespace['id']:
+            tags.append(tag)
+
+    return tags
+
+
+@log_call
+def metadef_tag_create(context, namespace_name, values):
+    """Create a metadef tag"""
+    global DATA
+
+    tag_values = copy.deepcopy(values)
+    tag_name = tag_values['name']
+    required_attributes = ['name']
+    allowed_attributes = ['name']
+
+    namespace = metadef_namespace_get(context, namespace_name)
+
+    for tag in DATA['metadef_tags']:
+        if tag['name'] == tag_name and tag['namespace_id'] == namespace['id']:
+            msg = ("A metadata definition tag with name=%(name)s"
+                   " in namespace=%(namespace_name)s already exists."
+                   % {'name': tag_name, 'namespace_name': namespace_name})
+            LOG.debug(msg)
+            raise exception.MetadefDuplicateTag(
+                name=tag_name, namespace_name=namespace_name)
+
+    for key in required_attributes:
+        if key not in tag_values:
+            raise exception.Invalid('%s is a required attribute' % key)
+
+    incorrect_keys = set(tag_values.keys()) - set(allowed_attributes)
+    if incorrect_keys:
+        raise exception.Invalid(
+            'The keys %s are not valid' % str(incorrect_keys))
+
+    tag_values['namespace_id'] = namespace['id']
+
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    tag = _format_tag(tag_values)
+    DATA['metadef_tags'].append(tag)
+    return tag
+
+
+@log_call
+def metadef_tag_create_tags(context, namespace_name, tag_list):
+    """Create a metadef tag"""
+    global DATA
+
+    namespace = metadef_namespace_get(context, namespace_name)
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    required_attributes = ['name']
+    allowed_attributes = ['name']
+    data_tag_list = []
+    tag_name_list = []
+    for tag_value in tag_list:
+        tag_values = copy.deepcopy(tag_value)
+        tag_name = tag_values['name']
+
+        for key in required_attributes:
+            if key not in tag_values:
+                raise exception.Invalid('%s is a required attribute' % key)
+
+        incorrect_keys = set(tag_values.keys()) - set(allowed_attributes)
+        if incorrect_keys:
+            raise exception.Invalid(
+                'The keys %s are not valid' % str(incorrect_keys))
+
+        if tag_name in tag_name_list:
+            msg = ("A metadata definition tag with name=%(name)s"
+                   " in namespace=%(namespace_name)s already exists."
+                   % {'name': tag_name, 'namespace_name': namespace_name})
+            LOG.debug(msg)
+            raise exception.MetadefDuplicateTag(
+                name=tag_name, namespace_name=namespace_name)
+        else:
+            tag_name_list.append(tag_name)
+
+        tag_values['namespace_id'] = namespace['id']
+        data_tag_list.append(_format_tag(tag_values))
+
+    DATA['metadef_tags'] = []
+    for tag in data_tag_list:
+        DATA['metadef_tags'].append(tag)
+
+    return data_tag_list
+
+
+@log_call
+def metadef_tag_update(context, namespace_name, id, values):
+    """Update a metadef tag"""
+    global DATA
+
+    namespace = metadef_namespace_get(context, namespace_name)
+
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    tag = metadef_tag_get_by_id(context, namespace_name, id)
+    if tag['name'] != values['name']:
+        for db_tag in DATA['metadef_tags']:
+            if (db_tag['name'] == values['name'] and
+                    db_tag['namespace_id'] == namespace['id']):
+                msg = ("Invalid update. It would result in a duplicate"
+                       " metadata definition tag with same name=%(name)s "
+                       " in namespace=%(namespace_name)s."
+                       % {'name': tag['name'],
+                          'namespace_name': namespace_name})
+                LOG.debug(msg)
+                raise exception.MetadefDuplicateTag(
+                    name=tag['name'], namespace_name=namespace_name)
+
+    DATA['metadef_tags'].remove(tag)
+
+    tag.update(values)
+    tag['updated_at'] = timeutils.utcnow()
+    DATA['metadef_tags'].append(tag)
+    return tag
+
+
+@log_call
+def metadef_tag_delete(context, namespace_name, name):
+    """Delete a metadef tag"""
+    global DATA
+
+    tags = metadef_tag_get(context, namespace_name, name)
+    DATA['metadef_tags'].remove(tags)
+
+    return tags
+
+
+def metadef_tag_delete_namespace_content(context, namespace_name,
+                                         session=None):
+    """Delete an tag or raise if namespace or tag doesn't exist."""
+    return _metadef_delete_namespace_content(
+        metadef_tag_get_all, 'metadef_tags', context, namespace_name)
+
+
+@log_call
+def metadef_tag_count(context, namespace_name):
+    """Get metadef tag count in a namespace"""
+    namespace = metadef_namespace_get(context, namespace_name)
+
+    _check_namespace_visibility(context, namespace, namespace_name)
+
+    count = 0
+    for tag in DATA['metadef_tags']:
+        if tag['namespace_id'] == namespace['id']:
+            count = count + 1
+
+    return count
+
+
+def _artifact_format(artifact_id, **values):
+    dt = timeutils.utcnow()
+    artifact = {
+        'id': artifact_id,
+        'type_name': None,
+        'type_version_prefix': None,
+        'type_version_suffix': None,
+        'type_version_meta': None,
+        'version_prefix': None,
+        'version_suffix': None,
+        'version_meta': None,
+        'description': None,
+        'visibility': None,
+        'state': None,
+        'owner': None,
+        'scope': None,
+        'tags': [],
+        'properties': {},
+        'blobs': [],
+        'created_at': dt,
+        'updated_at': dt,
+        'deleted_at': None,
+        'deleted': False,
+    }
+
+    artifact.update(values)
+    return artifact
+
+
+@log_call
+def artifact_create(context, values, type_name, type_version):
+    global DATA
+    artifact_id = values.get('id', str(uuid.uuid4()))
+
+    if artifact_id in DATA['artifacts']:
+        raise exception.Duplicate()
+
+    if 'state' not in values:
+        raise exception.Invalid('state is a required attribute')
+
+    allowed_keys = set(['id',
+                        'type_name',
+                        'type_version',
+                        'name',
+                        'version',
+                        'description',
+                        'visibility',
+                        'state',
+                        'owner',
+                        'scope'])
+
+    incorrect_keys = set(values.keys()) - allowed_keys
+    if incorrect_keys:
+        raise exception.Invalid(
+            'The keys %s are not valid' % str(incorrect_keys))
+
+    artifact = _artifact_format(artifact_id, **values)
+    DATA['artifacts'][artifact_id] = artifact
+
+    return copy.deepcopy(artifact)
+
+
+def _artifact_get(context, artifact_id, type_name,
+                  type_version=None):
+    try:
+        artifact = DATA['artifacts'][artifact_id]
+        if artifact['type_name'] != type_name or\
+                (type_version is not None and
+                 artifact['type_version'] != type_version):
+            raise KeyError
+    except KeyError:
+        LOG.info(_LI('Could not find artifact %s') % artifact_id)
+        raise exception.NotFound()
+
+    if artifact['deleted_at']:
+        LOG.info(_LI('Unable to get deleted image'))
+        raise exception.NotFound()
+
+    return artifact
+
+
+@log_call
+def artifact_get(context, artifact_id,
+                 type_name,
+                 type_version=None, session=None):
+    artifact = _artifact_get(context, artifact_id, type_name,
+                             type_version)
+    return copy.deepcopy(artifact)
+
+
 def _format_association(namespace, resource_type, association_values):
     association = {
         'namespace_id': namespace['id'],
@@ -1729,6 +2089,19 @@ def _format_object(values):
     return object
 
 
+def _format_tag(values):
+    dt = timeutils.utcnow()
+    tag = {
+        'id': _get_metadef_id(),
+        'namespace_id': None,
+        'name': None,
+        'created_at': dt,
+        'updated_at': dt
+    }
+    tag.update(values)
+    return tag
+
+
 def _is_namespace_visible(context, namespace):
     """Return true if namespace is visible in this context"""
     if context.is_admin:
@@ -1750,10 +2123,10 @@ def _is_namespace_visible(context, namespace):
 def _check_namespace_visibility(context, namespace, namespace_name):
     if not _is_namespace_visible(context, namespace):
         msg = ("Forbidding request, metadata definition namespace=%s"
-               " not visible." % namespace_name)
+               " is not visible." % namespace_name)
         LOG.debug(msg)
         emsg = _("Forbidding request, metadata definition namespace=%s"
-                 " not visible.") % namespace_name
+                 " is not visible.") % namespace_name
         raise exception.MetadefForbidden(emsg)
 
 
