@@ -13,7 +13,8 @@
 #    under the License.
 
 
-from oslo.db import exception as db_exc
+from oslo_db import exception as db_exc
+from oslo_log import log as logging
 from sqlalchemy import func
 import sqlalchemy.orm as sa_orm
 
@@ -22,9 +23,9 @@ from glance.db.sqlalchemy.metadef_api import namespace as namespace_api
 import glance.db.sqlalchemy.metadef_api.utils as metadef_utils
 from glance.db.sqlalchemy import models_metadef as models
 from glance import i18n
-import glance.openstack.common.log as os_logging
 
-LOG = os_logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
+_ = i18n._
 _LW = i18n._LW
 
 
@@ -34,9 +35,10 @@ def _get(context, object_id, session):
             .filter_by(id=object_id)
         metadef_object = query.one()
     except sa_orm.exc.NoResultFound:
-        LOG.warn(_LW("Metadata definition object not found for id %s",
-                     object_id))
-        raise exc.MetadefRecordNotFound(record_type='object', id=object_id)
+        msg = (_("Metadata definition object not found for id=%s")
+               % object_id)
+        LOG.warn(msg)
+        raise exc.MetadefObjectNotFound(msg)
 
     return metadef_object
 
@@ -66,7 +68,7 @@ def get_all(context, namespace_name, session):
 
     md_objects_list = []
     for obj in md_objects:
-        md_objects_list.append(obj.as_dict())
+        md_objects_list.append(obj.to_dict())
     return md_objects_list
 
 
@@ -88,13 +90,13 @@ def create(context, namespace_name, values, session):
         raise exc.MetadefDuplicateObject(
             object_name=md_object.name, namespace_name=namespace_name)
 
-    return md_object.as_dict()
+    return md_object.to_dict()
 
 
 def get(context, namespace_name, name, session):
     md_object = _get_by_name(context, namespace_name, name, session)
 
-    return md_object.as_dict()
+    return md_object.to_dict()
 
 
 def update(context, namespace_name, object_id, values, session):
@@ -119,7 +121,7 @@ def update(context, namespace_name, object_id, values, session):
                 % {'name': md_object.name, 'namespace_name': namespace_name})
         raise exc.MetadefDuplicateObject(emsg)
 
-    return md_object.as_dict()
+    return md_object.to_dict()
 
 
 def delete(context, namespace_name, object_name, session):
@@ -129,7 +131,7 @@ def delete(context, namespace_name, object_name, session):
     session.delete(md_object)
     session.flush()
 
-    return md_object.as_dict()
+    return md_object.to_dict()
 
 
 def delete_namespace_content(context, namespace_id, session):
