@@ -23,9 +23,10 @@ from glance.common import exception as exc
 from glance.db.sqlalchemy.metadef_api import namespace as namespace_api
 import glance.db.sqlalchemy.metadef_api.utils as metadef_utils
 from glance.db.sqlalchemy import models_metadef as models
-from glance.i18n import _LW
+from glance import i18n
 
 LOG = logging.getLogger(__name__)
+_LW = i18n._LW
 
 
 def _get(context, id, session):
@@ -34,7 +35,7 @@ def _get(context, id, session):
         metadef_tag = query.one()
     except sa_orm.exc.NoResultFound:
         msg = (_LW("Metadata tag not found for id %s") % id)
-        LOG.warning(msg)
+        LOG.warn(msg)
         raise exc.MetadefTagNotFound(message=msg)
     return metadef_tag
 
@@ -46,9 +47,10 @@ def _get_by_name(context, namespace_name, name, session):
             name=name, namespace_id=namespace['id']))
         metadef_tag = query.one()
     except sa_orm.exc.NoResultFound:
-        LOG.debug("The metadata tag with name=%(name)s"
-                  " was not found in namespace=%(namespace_name)s.",
-                  {'name': name, 'namespace_name': namespace_name})
+        msg = ("The metadata tag with name=%(name)s"
+               " was not found in namespace=%(namespace_name)s."
+               % {'name': name, 'namespace_name': namespace_name})
+        LOG.debug(msg)
         raise exc.MetadefTagNotFound(name=name,
                                      namespace_name=namespace_name)
     return metadef_tag
@@ -99,19 +101,18 @@ def create(context, namespace_name, values, session):
     try:
         metadef_tag.save(session=session)
     except db_exc.DBDuplicateEntry:
-        LOG.debug("A metadata tag name=%(name)s"
-                  " already exists in namespace=%(namespace_name)s."
-                  " (Please note that metadata tag names are"
-                  " case insensitive).",
-                  {'name': metadef_tag.name,
-                   'namespace_name': namespace_name})
+        msg = ("A metadata tag name=%(name)s"
+               " in namespace=%(namespace_name)s already exists."
+               % {'name': metadef_tag.name,
+                  'namespace_name': namespace_name})
+        LOG.debug(msg)
         raise exc.MetadefDuplicateTag(
             name=metadef_tag.name, namespace_name=namespace_name)
 
     return metadef_tag.to_dict()
 
 
-def create_tags(context, namespace_name, tag_list, can_append, session):
+def create_tags(context, namespace_name, tag_list, session):
 
     metadef_tags_list = []
     if tag_list:
@@ -119,10 +120,10 @@ def create_tags(context, namespace_name, tag_list, can_append, session):
 
         try:
             with session.begin():
-                if not can_append:
-                    query = (session.query(models.MetadefTag).filter_by(
-                             namespace_id=namespace['id']))
-                    query.delete(synchronize_session='fetch')
+                query = (session.query(models.MetadefTag).filter_by(
+                    namespace_id=namespace['id']))
+                query.delete(synchronize_session='fetch')
+
                 for value in tag_list:
                     value.update({'namespace_id': namespace['id']})
                     metadef_utils.drop_protected_attrs(
@@ -132,10 +133,11 @@ def create_tags(context, namespace_name, tag_list, can_append, session):
                     metadef_tag.save(session=session)
                     metadef_tags_list.append(metadef_tag.to_dict())
         except db_exc.DBDuplicateEntry:
-            LOG.debug("A metadata tag name=%(name)s"
-                      " in namespace=%(namespace_name)s already exists.",
-                      {'name': metadef_tag.name,
-                       'namespace_name': namespace_name})
+            msg = ("A metadata tag name=%(name)s"
+                   " in namespace=%(namespace_name)s already exists."
+                   % {'name': metadef_tag.name,
+                      'namespace_name': namespace_name})
+            LOG.debug(msg)
             raise exc.MetadefDuplicateTag(
                 name=metadef_tag.name, namespace_name=namespace_name)
 
@@ -158,11 +160,12 @@ def update(context, namespace_name, id, values, session):
         metadata_tag.update(values.copy())
         metadata_tag.save(session=session)
     except db_exc.DBDuplicateEntry:
-        LOG.debug("Invalid update. It would result in a duplicate"
-                  " metadata tag with same name=%(name)s"
-                  " in namespace=%(namespace_name)s.",
-                  {'name': values['name'],
-                   'namespace_name': namespace_name})
+        msg = ("Invalid update. It would result in a duplicate"
+               " metadata tag with same name=%(name)s"
+               " in namespace=%(namespace_name)s."
+               % {'name': values['name'],
+                  'namespace_name': namespace_name})
+        LOG.debug(msg)
         raise exc.MetadefDuplicateTag(
             name=values['name'], namespace_name=namespace_name)
 
