@@ -17,14 +17,14 @@
 Various conveniences used for migration scripts
 """
 
+from oslo_log import log as logging
 import sqlalchemy.types
 
-from glance.openstack.common import gettextutils
-import glance.openstack.common.log as logging
+from glance import i18n
 
 
 LOG = logging.getLogger(__name__)
-_LI = gettextutils._LI
+_LI = i18n._LI
 
 
 String = lambda length: sqlalchemy.types.String(
@@ -52,6 +52,9 @@ BigInteger = lambda: sqlalchemy.types.BigInteger()
 PickleType = lambda: sqlalchemy.types.PickleType()
 
 
+Numeric = lambda: sqlalchemy.types.Numeric()
+
+
 def from_migration_import(module_name, fromlist):
     """
     Import a migration file and return the module
