@@ -14,14 +14,7 @@ import migrate
 import sqlalchemy
 from sqlalchemy import inspect
 from sqlalchemy import (Table, Index, UniqueConstraint)
-from sqlalchemy.schema import (DropConstraint)
-
-
-def _change_db2_unique_constraint(operation_type, constraint_name, *columns):
-    constraint = migrate.UniqueConstraint(*columns, name=constraint_name)
-
-    operation = getattr(constraint, operation_type)
-    operation()
+from sqlalchemy.schema import (AddConstraint, DropConstraint)
 
 
 def upgrade(migrate_engine):
@@ -38,37 +31,14 @@ def upgrade(migrate_engine):
                                    autoload=True)
     metadef_tags = Table('metadef_tags', meta, autoload=True)
 
-    constraints = [('ix_namespaces_namespace',
-                    [metadef_namespaces.c.namespace]),
-                   ('ix_objects_namespace_id_name',
-                    [metadef_objects.c.namespace_id,
-                     metadef_objects.c.name]),
-                   ('ix_metadef_properties_namespace_id_name',
-                    [metadef_properties.c.namespace_id,
-                     metadef_properties.c.name])]
-    metadef_tags_constraints = inspector.get_unique_constraints('metadef_tags')
-    for constraint in metadef_tags_constraints:
-        if set(constraint['column_names']) == set(['namespace_id', 'name']):
-            constraints.append((constraint['name'],
-                                [metadef_tags.c.namespace_id,
-                                 metadef_tags.c.name]))
-    if meta.bind.name == "ibm_db_sa":
-        # For db2, the following constraints need to be dropped first,
-        # otherwise the index like ix_metadef_ns_res_types_namespace_id
-        # will fail to create. These constraints will be added back at
-        # the end. It should not affect the origional logic for other
-        # database backends.
-        for (constraint_name, cols) in constraints:
-            _change_db2_unique_constraint('drop', constraint_name, *cols)
-    else:
-        Index('ix_namespaces_namespace', metadef_namespaces.c.namespace).drop()
+    Index('ix_namespaces_namespace', metadef_namespaces.c.namespace).drop()
 
-        Index('ix_objects_namespace_id_name', metadef_objects.c.namespace_id,
-              metadef_objects.c.name).drop()
+    Index('ix_objects_namespace_id_name', metadef_objects.c.namespace_id,
+          metadef_objects.c.name).drop()
 
-        Index('ix_metadef_properties_namespace_id_name',
-              metadef_properties.c.namespace_id,
-              metadef_properties.c.name).drop()
+    Index('ix_metadef_properties_namespace_id_name',
+          metadef_properties.c.namespace_id,
+          metadef_properties.c.name).drop()
 
     fkc = migrate.ForeignKeyConstraint([metadef_tags.c.namespace_id],
                                        [metadef_namespaces.c.id])
@@ -87,9 +57,8 @@ def upgrade(migrate_engine):
                                       metadef_tags.c.name)
         uc.create()
 
-    if meta.bind.name != "ibm_db_sa":
-        Index('ix_tags_namespace_id_name', metadef_tags.c.namespace_id,
-              metadef_tags.c.name).drop()
+    Index('ix_tags_namespace_id_name', metadef_tags.c.namespace_id,
+          metadef_tags.c.name).drop()
 
     Index('ix_metadef_tags_name', metadef_tags.c.name).create()
 
@@ -189,8 +158,138 @@ def upgrade(migrate_engine):
     Index('ix_metadef_properties_namespace_id',
           metadef_properties.c.namespace_id).create()
 
-    if meta.bind.name == "ibm_db_sa":
-        # For db2, add these constraints back. It should not affect the
-        # origional logic for other database backends.
-        for (constraint_name, cols) in constraints:
-            _change_db2_unique_constraint('create', constraint_name, *cols)
+
+def downgrade(migrate_engine):
+    meta = sqlalchemy.MetaData()
+    meta.bind = migrate_engine
+
+    metadef_namespaces = Table('metadef_namespaces', meta, autoload=True)
+    metadef_properties = Table('metadef_properties', meta, autoload=True)
+    metadef_objects = Table('metadef_objects', meta, autoload=True)
+    metadef_ns_res_types = Table('metadef_namespace_resource_types',
+                                 meta, autoload=True)
+    metadef_resource_types = Table('metadef_resource_types', meta,
+                                   autoload=True)
+    metadef_tags = Table('metadef_tags', meta, autoload=True)
+
+    Index('ix_namespaces_namespace', metadef_namespaces.c.namespace).create()
+
+    Index('ix_objects_namespace_id_name', metadef_objects.c.namespace_id,
+          metadef_objects.c.name).create()
+
+    Index('ix_metadef_properties_namespace_id_name',
+          metadef_properties.c.namespace_id,
+          metadef_properties.c.name).create()
+
+    Index('ix_metadef_tags_name', metadef_tags.c.name).drop()
+
+    Index('ix_metadef_tags_namespace_id', metadef_tags.c.namespace_id,
+          metadef_tags.c.name).drop()
+
+    if migrate_engine.name != 'sqlite':
+        fkc = migrate.ForeignKeyConstraint([metadef_tags.c.namespace_id],
+                                           [metadef_namespaces.c.id])
+        fkc.drop()
+
+        Index('ix_tags_namespace_id_name', metadef_tags.c.namespace_id,
+              metadef_tags.c.name).create()
+    else:
+        # NOTE(ochuprykov): fkc can't be dropped via `migrate` in sqlite,so it
+        # is necessary to recreate table manually and populate it with data
+        temp = Table('temp_', meta, *(
+            [c.copy() for c in metadef_tags.columns]))
+        temp.create()
+        migrate_engine.execute('insert into temp_ select * from metadef_tags')
+        metadef_tags.drop()
+        migrate_engine.execute('alter table temp_ rename to metadef_tags')
+
+        # Refresh old metadata for this table
+        meta = sqlalchemy.MetaData()
+        meta.bind = migrate_engine
+        metadef_tags = Table('metadef_tags', meta, autoload=True)
+
+        Index('ix_tags_namespace_id_name', metadef_tags.c.namespace_id,
+              metadef_tags.c.name).create()
+
+        uc = migrate.UniqueConstraint(metadef_tags.c.namespace_id,
+                                      metadef_tags.c.name)
+        uc.create()
+
+    if migrate_engine.name == 'mysql':
+        constraint = UniqueConstraint(metadef_properties.c.namespace_id,
+                                      metadef_properties.c.name,
+                                      name='namespace_id')
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(metadef_objects.c.namespace_id,
+                                      metadef_objects.c.name,
+                                      name='namespace_id')
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(metadef_ns_res_types.c.resource_type_id,
+                                      metadef_ns_res_types.c.namespace_id,
+                                      name='resource_type_id')
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(metadef_namespaces.c.namespace,
+                                      name='namespace')
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(metadef_resource_types.c.name,
+                                      name='name')
+        migrate_engine.execute(AddConstraint(constraint))
+
+    if migrate_engine.name == 'postgresql':
+        constraint = UniqueConstraint(
+            metadef_objects.c.namespace_id,
+            metadef_objects.c.name)
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(
+            metadef_properties.c.namespace_id,
+            metadef_properties.c.name)
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(
+            metadef_namespaces.c.namespace)
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(
+            metadef_resource_types.c.name)
+        migrate_engine.execute(AddConstraint(constraint))
+
+        constraint = UniqueConstraint(
+            metadef_tags.c.namespace_id,
+            metadef_tags.c.name,
+            name='metadef_tags_namespace_id_name_key')
+        migrate_engine.execute(AddConstraint(constraint))
+
+    if migrate_engine.name == 'mysql':
+        fkc = migrate.ForeignKeyConstraint(
+            [metadef_ns_res_types.c.resource_type_id],
+            [metadef_namespaces.c.id],
+            name='metadef_namespace_resource_types_ibfk_2')
+        fkc.drop()
+
+        Index('ix_metadef_ns_res_types_namespace_id',
+              metadef_ns_res_types.c.namespace_id).drop()
+
+        fkc.create()
+    else:
+        Index('ix_metadef_ns_res_types_namespace_id',
+              metadef_ns_res_types.c.namespace_id).drop()
+
+    Index('ix_metadef_namespaces_namespace',
+          metadef_namespaces.c.namespace).drop()
+
+    Index('ix_metadef_namespaces_owner', metadef_namespaces.c.owner).drop()
+
+    Index('ix_metadef_objects_name', metadef_objects.c.name).drop()
+
+    Index('ix_metadef_objects_namespace_id',
+          metadef_objects.c.namespace_id).drop()
+
+    Index('ix_metadef_properties_name', metadef_properties.c.name).drop()
+
+    Index('ix_metadef_properties_namespace_id',
+          metadef_properties.c.namespace_id).drop()
