@@ -13,10 +13,13 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import glance_store
+from oslo_log import log as logging
 
 from glance.api import authorization
 from glance.api import policy
 from glance.api import property_protections
+from glance.common import exception
 from glance.common import property_utils
 from glance.common import store_utils
 import glance.db
@@ -24,17 +27,28 @@ import glance.domain
 import glance.location
 import glance.notifier
 import glance.quota
-import glance_store
+try:
+    import glance.search
+    glance_search = glance.search
+except ImportError:
+    glance_search = None
+
+
+LOG = logging.getLogger(__name__)
 
 
 class Gateway(object):
     def __init__(self, db_api=None, store_api=None, notifier=None,
-                 policy_enforcer=None):
+                 policy_enforcer=None, es_api=None):
         self.db_api = db_api or glance.db.get_api()
         self.store_api = store_api or glance_store
         self.store_utils = store_utils
         self.notifier = notifier or glance.notifier.Notifier()
         self.policy = policy_enforcer or policy.Enforcer()
+        if es_api:
+            self.es_api = es_api
+        else:
+            self.es_api = glance_search.get_api() if glance_search else None
 
     def get_image_factory(self, context):
         image_factory = glance.domain.ImageFactory()
@@ -48,11 +62,10 @@ class Gateway(object):
             policy_image_factory, context, self.notifier)
         if property_utils.is_property_protection_enabled():
             property_rules = property_utils.PropertyRules(self.policy)
-            protected_image_factory = property_protections.\
-                ProtectedImageFactoryProxy(notifier_image_factory, context,
-                                           property_rules)
+            pif = property_protections.ProtectedImageFactoryProxy(
+                notifier_image_factory, context, property_rules)
             authorized_image_factory = authorization.ImageFactoryProxy(
-                protected_image_factory, context)
+                pif, context)
         else:
             authorized_image_factory = authorization.ImageFactoryProxy(
                 notifier_image_factory, context)
@@ -80,11 +93,10 @@ class Gateway(object):
             policy_image_repo, context, self.notifier)
         if property_utils.is_property_protection_enabled():
             property_rules = property_utils.PropertyRules(self.policy)
-            protected_image_repo = property_protections.\
-                ProtectedImageRepoProxy(notifier_image_repo, context,
-                                        property_rules)
+            pir = property_protections.ProtectedImageRepoProxy(
+                notifier_image_repo, context, property_rules)
             authorized_image_repo = authorization.ImageRepoProxy(
-                protected_image_repo, context)
+                pir, context)
         else:
             authorized_image_repo = authorization.ImageRepoProxy(
                 notifier_image_repo, context)
@@ -133,41 +145,54 @@ class Gateway(object):
         ns_factory = glance.domain.MetadefNamespaceFactory()
         policy_ns_factory = policy.MetadefNamespaceFactoryProxy(
             ns_factory, context, self.policy)
+        notifier_ns_factory = glance.notifier.MetadefNamespaceFactoryProxy(
+            policy_ns_factory, context, self.notifier)
         authorized_ns_factory = authorization.MetadefNamespaceFactoryProxy(
-            policy_ns_factory, context)
+            notifier_ns_factory, context)
         return authorized_ns_factory
 
     def get_metadef_namespace_repo(self, context):
         ns_repo = glance.db.MetadefNamespaceRepo(context, self.db_api)
         policy_ns_repo = policy.MetadefNamespaceRepoProxy(
             ns_repo, context, self.policy)
+        notifier_ns_repo = glance.notifier.MetadefNamespaceRepoProxy(
+            policy_ns_repo, context, self.notifier)
         authorized_ns_repo = authorization.MetadefNamespaceRepoProxy(
-            policy_ns_repo, context)
+            notifier_ns_repo, context)
         return authorized_ns_repo
 
     def get_metadef_object_factory(self, context):
         object_factory = glance.domain.MetadefObjectFactory()
         policy_object_factory = policy.MetadefObjectFactoryProxy(
             object_factory, context, self.policy)
+        notifier_object_factory = glance.notifier.MetadefObjectFactoryProxy(
+            policy_object_factory, context, self.notifier)
         authorized_object_factory = authorization.MetadefObjectFactoryProxy(
-            policy_object_factory, context)
+            notifier_object_factory, context)
         return authorized_object_factory
 
     def get_metadef_object_repo(self, context):
         object_repo = glance.db.MetadefObjectRepo(context, self.db_api)
         policy_object_repo = policy.MetadefObjectRepoProxy(
             object_repo, context, self.policy)
+        notifier_object_repo = glance.notifier.MetadefObjectRepoProxy(
+            policy_object_repo, context, self.notifier)
         authorized_object_repo = authorization.MetadefObjectRepoProxy(
-            policy_object_repo, context)
+            notifier_object_repo, context)
         return authorized_object_repo
 
     def get_metadef_resource_type_factory(self, context):
         resource_type_factory = glance.domain.MetadefResourceTypeFactory()
         policy_resource_type_factory = policy.MetadefResourceTypeFactoryProxy(
             resource_type_factory, context, self.policy)
-        authorized_resource_type_factory = \
+        notifier_resource_type_factory = (
+            glance.notifier.MetadefResourceTypeFactoryProxy(
+                policy_resource_type_factory, context, self.notifier)
+        )
+        authorized_resource_type_factory = (
             authorization.MetadefResourceTypeFactoryProxy(
-                policy_resource_type_factory, context)
+                notifier_resource_type_factory, context)
+        )
         return authorized_resource_type_factory
 
     def get_metadef_resource_type_repo(self, context):
@@ -175,23 +200,63 @@ class Gateway(object):
             context, self.db_api)
         policy_object_repo = policy.MetadefResourceTypeRepoProxy(
             resource_type_repo, context, self.policy)
-        authorized_resource_type_repo = \
-            authorization.MetadefResourceTypeRepoProxy(policy_object_repo,
-                                                       context)
-        return authorized_resource_type_repo
+        notifier_object_repo = glance.notifier.MetadefResourceTypeRepoProxy(
+            policy_object_repo, context, self.notifier)
+        authorized_object_repo = authorization.MetadefResourceTypeRepoProxy(
+            notifier_object_repo, context)
+        return authorized_object_repo
 
     def get_metadef_property_factory(self, context):
         prop_factory = glance.domain.MetadefPropertyFactory()
         policy_prop_factory = policy.MetadefPropertyFactoryProxy(
             prop_factory, context, self.policy)
+        notifier_prop_factory = glance.notifier.MetadefPropertyFactoryProxy(
+            policy_prop_factory, context, self.notifier)
         authorized_prop_factory = authorization.MetadefPropertyFactoryProxy(
-            policy_prop_factory, context)
+            notifier_prop_factory, context)
         return authorized_prop_factory
 
     def get_metadef_property_repo(self, context):
         prop_repo = glance.db.MetadefPropertyRepo(context, self.db_api)
         policy_prop_repo = policy.MetadefPropertyRepoProxy(
             prop_repo, context, self.policy)
+        notifier_prop_repo = glance.notifier.MetadefPropertyRepoProxy(
+            policy_prop_repo, context, self.notifier)
         authorized_prop_repo = authorization.MetadefPropertyRepoProxy(
-            policy_prop_repo, context)
+            notifier_prop_repo, context)
         return authorized_prop_repo
+
+    def get_metadef_tag_factory(self, context):
+        tag_factory = glance.domain.MetadefTagFactory()
+        policy_tag_factory = policy.MetadefTagFactoryProxy(
+            tag_factory, context, self.policy)
+        notifier_tag_factory = glance.notifier.MetadefTagFactoryProxy(
+            policy_tag_factory, context, self.notifier)
+        authorized_tag_factory = authorization.MetadefTagFactoryProxy(
+            notifier_tag_factory, context)
+        return authorized_tag_factory
+
+    def get_metadef_tag_repo(self, context):
+        tag_repo = glance.db.MetadefTagRepo(context, self.db_api)
+        policy_tag_repo = policy.MetadefTagRepoProxy(
+            tag_repo, context, self.policy)
+        notifier_tag_repo = glance.notifier.MetadefTagRepoProxy(
+            policy_tag_repo, context, self.notifier)
+        authorized_tag_repo = authorization.MetadefTagRepoProxy(
+            notifier_tag_repo, context)
+        return authorized_tag_repo
+
+    def get_catalog_search_repo(self, context):
+        if self.es_api is None:
+            # TODO(mriedem): Make this a separate exception or change to
+            # warning/error logging in Liberty once we're past string freeze.
+            # See bug 1441764.
+            LOG.debug('The search and index services are not available. '
+                      'Ensure you have the necessary prerequisite '
+                      'dependencies installed like elasticsearch to use these '
+                      'services.')
+            raise exception.ServiceUnavailable()
+        search_repo = glance.search.CatalogSearchRepo(context, self.es_api)
+        policy_search_repo = policy.CatalogSearchRepoProxy(
+            search_repo, context, self.policy)
+        return policy_search_repo
