@@ -19,13 +19,15 @@ Base attribute driver class
 
 import os.path
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
 
 from glance.common import exception
 from glance.common import utils
-import glance.openstack.common.log as logging
+from glance import i18n
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
 
 CONF = cfg.CONF
 
