@@ -17,17 +17,18 @@ import collections
 import copy
 
 import glance_store as store
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import excutils
 
 from glance.common import exception
 from glance.common import utils
 import glance.domain.proxy
-from glance.openstack.common import excutils
-from glance.openstack.common import gettextutils
-import glance.openstack.common.log as logging
+from glance import i18n
 
 
-_LE = gettextutils._LE
+_ = i18n._
+_LE = i18n._LE
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -115,7 +116,7 @@ def _count_duplicated_locations(locations, new):
 
     ret = 0
     for loc in locations:
-        if (loc['url'] == new['url'] and loc['metadata'] == new['metadata']):
+        if loc['url'] == new['url'] and loc['metadata'] == new['metadata']:
             ret += 1
     return ret
 
@@ -382,7 +383,12 @@ class ImageProxy(glance.domain.proxy.Image):
 
     def get_data(self, offset=0, chunk_size=None):
         if not self.image.locations:
-            raise store.NotFound(_("No image data could be found"))
+            # NOTE(mclaren): This is the only set of arguments
+            # which work with this exception currently, see:
+            # https://bugs.launchpad.net/glance-store/+bug/1501443
+            # When the above glance_store bug is fixed we can
+            # add a msg as usual.
+            raise store.NotFound(image=None)
         err = None
         for loc in self.image.locations:
             try:
