@@ -19,13 +19,15 @@ Registry's Client V2
 
 import os
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
 
 from glance.common import exception
-import glance.openstack.common.log as logging
+from glance import i18n
 from glance.registry.client.v2 import client
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
 
 CONF = cfg.CONF
 _registry_client = 'glance.registry.client'
@@ -93,7 +95,7 @@ def configure_registry_admin_creds():
         'password': CONF.admin_password,
         'username': CONF.admin_user,
         'tenant': CONF.admin_tenant_name,
-        'auth_url': CONF.auth_url,
+        'auth_url': os.getenv('OS_AUTH_URL') or CONF.auth_url,
         'strategy': strategy,
         'region': CONF.auth_region,
     }
