@@ -18,6 +18,9 @@ import six
 
 from glance.common import exception
 from glance.common import utils
+from glance import i18n
+
+_ = i18n._
 
 
 class Schema(object):
@@ -42,8 +45,24 @@ class Schema(object):
     def filter(self, obj):
         filtered = {}
         for key, value in six.iteritems(obj):
-            if self._filter_func(self.properties, key) and value is not None:
+            if self._filter_func(self.properties, key):
                 filtered[key] = value
+
+            # NOTE(flaper87): This exists to allow for v1, null properties,
+            # to be used with the V2 API. During Kilo, it was allowed for the
+            # later to return None values without considering that V1 allowed
+            # for custom properties to be None, which is something V2 doesn't
+            # allow for. This small hack here will set V1 custom `None` pro-
+            # perties to an empty string so that they will be updated along
+            # with the image (if an update happens).
+            #
+            # We could skip the properties that are `None` but that would bring
+            # back the behavior we moved away from. Note that we can't consider
+            # doing a schema migration because we don't know which properties
+            # are "custom" and which came from `schema-image` if those custom
+            # properties were created with v1.
+            if key not in self.properties and value is None:
+                filtered[key] = ''
         return filtered
 
     @staticmethod
