@@ -13,10 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_policy import policy
+# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
+from six.moves import range
 import testtools
 import webob
 
 import glance.api.middleware.cache
+import glance.api.policy
 from glance.common import exception
 from glance import context
 import glance.registry.client.v1.api as registry
@@ -48,12 +52,12 @@ class TestCacheMiddlewareURLMatching(testtools.TestCase):
     def test_v1_match_id_with_query_param(self):
         req = webob.Request.blank('/v1/images/asdf?ping=pong')
         out = glance.api.middleware.cache.CacheFilter._match_request(req)
-        self.assertEqual(out, ('v1', 'GET', 'asdf'))
+        self.assertEqual(('v1', 'GET', 'asdf'), out)
 
     def test_v2_match_id(self):
         req = webob.Request.blank('/v2/images/asdf/file')
         out = glance.api.middleware.cache.CacheFilter._match_request(req)
-        self.assertEqual(out, ('v2', 'GET', 'asdf'))
+        self.assertEqual(('v2', 'GET', 'asdf'), out)
 
     def test_v2_no_match_bad_path(self):
         req = webob.Request.blank('/v2/images/asdf')
@@ -74,9 +78,9 @@ class TestCacheMiddlewareRequestStashCacheInfo(testtools.TestCase):
 
     def test_stash_cache_request_info(self):
         self.middleware._stash_request_info(self.request, 'asdf', 'GET', 'v2')
-        self.assertEqual(self.request.environ['api.cache.image_id'], 'asdf')
-        self.assertEqual(self.request.environ['api.cache.method'], 'GET')
-        self.assertEqual(self.request.environ['api.cache.version'], 'v2')
+        self.assertEqual('asdf', self.request.environ['api.cache.image_id'])
+        self.assertEqual('GET', self.request.environ['api.cache.method'])
+        self.assertEqual('v2', self.request.environ['api.cache.version'])
 
     def test_fetch_cache_request_info(self):
         self.request.environ['api.cache.image_id'] = 'asdf'
@@ -167,6 +171,12 @@ class ProcessRequestTestCacheFilter(glance.api.middleware.cache.CacheFilter):
 
 
 class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
+    def _enforcer_from_rules(self, unparsed_rules):
+        rules = policy.Rules.from_dict(unparsed_rules)
+        enforcer = glance.api.policy.Enforcer()
+        enforcer.set_rules(rules, overwrite=True)
+        return enforcer
+
     def test_v1_deleted_image_fetch(self):
         """
         Test for determining that when an admin tries to download a deleted
@@ -213,7 +223,7 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
             raise exception.NotFound()
 
         def fake_get_v1_image_metadata(request, image_id):
-            return {'properties': {}}
+            return {'status': 'active', 'properties': {}}
 
         image_id = 'test1'
         request = webob.Request.blank('/v1/images/%s' % image_id)
@@ -376,7 +386,7 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
         """
 
         def fake_get_v1_image_metadata(*args, **kwargs):
-            return {'properties': {}}
+            return {'status': 'active', 'properties': {}}
 
         image_id = 'test1'
         request = webob.Request.blank('/v1/images/%s' % image_id)
@@ -385,9 +395,8 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
         cache_filter = ProcessRequestTestCacheFilter()
         cache_filter._get_v1_image_metadata = fake_get_v1_image_metadata
 
-        rules = {'download_image': '!'}
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
+        enforcer = self._enforcer_from_rules({'download_image': '!'})
+        cache_filter.policy = enforcer
         self.assertRaises(webob.exc.HTTPForbidden,
                           cache_filter.process_request, request)
 
@@ -420,18 +429,17 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
                 'x_test_key': 'test_1234'
             }
 
+        enforcer = self._enforcer_from_rules({
+            "restricted":
+            "not ('test_1234':%(x_test_key)s and role:_member_)",
+            "download_image": "role:admin or rule:restricted"
+        })
+
         request = webob.Request.blank('/v1/images/%s' % image_id)
         request.context = context.RequestContext(roles=['_member_'])
         cache_filter = ProcessRequestTestCacheFilter()
         cache_filter._get_v1_image_metadata = fake_get_v1_image_metadata
-
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
+        cache_filter.policy = enforcer
         self.assertRaises(webob.exc.HTTPForbidden,
                           cache_filter.process_request, request)
 
@@ -520,18 +528,18 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
             request.environ['api.cache.image'] = image
             return glance.api.policy.ImageTarget(image)
 
+        enforcer = self._enforcer_from_rules({
+            "restricted":
+            "not ('test_1234':%(x_test_key)s and role:_member_)",
+            "download_image": "role:admin or rule:restricted"
+        })
+
         request = webob.Request.blank('/v2/images/test1/file')
         request.context = context.RequestContext(roles=['_member_'])
         cache_filter = ProcessRequestTestCacheFilter()
         cache_filter._get_v2_image_metadata = fake_get_v2_image_metadata
 
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
+        cache_filter.policy = enforcer
         self.assertRaises(webob.exc.HTTPForbidden,
                           cache_filter.process_request, request)
 
@@ -565,76 +573,6 @@ class TestCacheMiddlewareProcessRequest(base.IsolatedUnitTest):
         actual = cache_filter.process_request(request)
         self.assertTrue(actual)
 
-    def test_v1_process_request_download_without_restricted_property(self):
-        """
-        Test process_request for v1 api where _member_ role able to
-        download the image if restricted property is not added to the image.
-        """
-        image_id = 'test1'
-
-        def fake_get_v1_image_metadata(*args, **kwargs):
-            return {
-                'id': image_id,
-                'name': 'fake_image',
-                'status': 'active',
-                'created_at': '',
-                'min_disk': '10G',
-                'min_ram': '1024M',
-                'protected': False,
-                'locations': '',
-                'checksum': 'c1234',
-                'owner': '',
-                'disk_format': 'raw',
-                'container_format': 'bare',
-                'size': '123456789',
-                'virtual_size': '123456789',
-                'is_public': 'public',
-                'deleted': False,
-                'updated_at': ''
-            }
-
-        request = webob.Request.blank('/v1/images/%s' % image_id)
-        request.context = context.RequestContext(roles=['_member_'])
-        cache_filter = ProcessRequestTestCacheFilter()
-        cache_filter._get_v1_image_metadata = fake_get_v1_image_metadata
-
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
-        actual = cache_filter.process_request(request)
-        self.assertTrue(actual)
-
-    def test_v2_process_request_download_without_restricted_property(self):
-        """
-        Test process_request for v2 api where member role able to
-        download the image if restricted property is not added to the image.
-        """
-        image_id = 'test1'
-
-        def fake_get_v2_image_metadata(*args, **kwargs):
-            image = ImageStub(image_id)
-            request.environ['api.cache.image'] = image
-            return glance.api.policy.ImageTarget(image)
-
-        request = webob.Request.blank('/v2/images/test1/file')
-        request.context = context.RequestContext(roles=['_member_'])
-        cache_filter = ProcessRequestTestCacheFilter()
-        cache_filter._get_v2_image_metadata = fake_get_v2_image_metadata
-
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
-        actual = cache_filter.process_request(request)
-        self.assertTrue(actual)
-
 
 class TestCacheMiddlewareProcessResponse(base.IsolatedUnitTest):
     def test_process_v1_DELETE_response(self):
@@ -645,7 +583,7 @@ class TestCacheMiddlewareProcessResponse(base.IsolatedUnitTest):
         headers = {"x-image-meta-deleted": True}
         resp = webob.Response(request=request, headers=headers)
         actual = cache_filter._process_DELETE_response(resp, image_id)
-        self.assertEqual(actual, resp)
+        self.assertEqual(resp, actual)
 
     def test_get_status_code(self):
         headers = {"x-image-meta-deleted": True}
@@ -670,7 +608,7 @@ class TestCacheMiddlewareProcessResponse(base.IsolatedUnitTest):
         headers = {"x-image-meta-deleted": True}
         resp = webob.Response(request=request, headers=headers)
         actual = cache_filter.process_response(resp)
-        self.assertEqual(actual, resp)
+        self.assertEqual(resp, actual)
 
     def test_process_response_without_download_image_policy(self):
         """
@@ -900,84 +838,3 @@ class TestCacheMiddlewareProcessResponse(base.IsolatedUnitTest):
         resp = webob.Response(request=request)
         actual = cache_filter.process_response(resp)
         self.assertEqual(actual, resp)
-
-    def test_v1_process_response_download_without_restricted_property(self):
-        """
-        Test process_response for v1 api where _member_ role able to
-        download the image if restricted property is not added to the image.
-        """
-        image_id = 'test1'
-
-        def fake_fetch_request_info(*args, **kwargs):
-            return ('test1', 'GET', 'v1')
-
-        def fake_get_v1_image_metadata(*args, **kwargs):
-            return {
-                'id': image_id,
-                'name': 'fake_image',
-                'status': 'active',
-                'created_at': '',
-                'min_disk': '10G',
-                'min_ram': '1024M',
-                'protected': False,
-                'locations': '',
-                'checksum': 'c1234',
-                'owner': '',
-                'disk_format': 'raw',
-                'container_format': 'bare',
-                'size': '123456789',
-                'virtual_size': '123456789',
-                'is_public': 'public',
-                'deleted': False,
-                'updated_at': ''
-            }
-
-        cache_filter = ProcessRequestTestCacheFilter()
-        cache_filter._fetch_request_info = fake_fetch_request_info
-        cache_filter._get_v1_image_metadata = fake_get_v1_image_metadata
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
-
-        request = webob.Request.blank('/v1/images/%s' % image_id)
-        request.context = context.RequestContext(roles=['_member_'])
-        resp = webob.Response(request=request)
-        actual = cache_filter.process_response(resp)
-        self.assertEqual(actual, resp)
-
-    def test_v2_process_response_download_without_restricted_property(self):
-        """
-        Test process_response for v2 api where member role able to
-        download the image if restricted property is not added to the image.
-        """
-        image_id = 'test1'
-
-        def fake_fetch_request_info(*args, **kwargs):
-            return ('test1', 'GET', 'v2')
-
-        def fake_get_v2_image_metadata(*args, **kwargs):
-            image = ImageStub(image_id)
-            request.environ['api.cache.image'] = image
-            return glance.api.policy.ImageTarget(image)
-
-        cache_filter = ProcessRequestTestCacheFilter()
-        cache_filter._fetch_request_info = fake_fetch_request_info
-        cache_filter._get_v2_image_metadata = fake_get_v2_image_metadata
-
-        rules = {
-            "restricted":
-            "not ('test_1234':%(x_test_key)s and role:_member_)",
-            "download_image": "role:admin or rule:restricted"
-        }
-        self.set_policy_rules(rules)
-        cache_filter.policy = glance.api.policy.Enforcer()
-
-        request = webob.Request.blank('/v2/images/test1/file')
-        request.context = context.RequestContext(roles=['_member_'])
-        resp = webob.Response(request=request)
-        actual = cache_filter.process_response(resp)
-        self.assertEqual(actual, resp)
