@@ -12,16 +12,17 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import uuid
 
 import mock
 from mock import patch
-import uuid
-
+from oslo_utils import units
 import six
+# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
+from six.moves import range
 
 from glance.common import exception
 from glance.common import store_utils
-from glance.openstack.common import units
 import glance.quota
 from glance.tests.unit import utils as unit_test_utils
 from glance.tests import utils as test_utils
@@ -145,7 +146,7 @@ class TestImageQuota(test_utils.BaseTestCase):
     def test_quota_exceeded_no_size(self):
         quota = 10
         data = '*' * (quota + 1)
-        #NOTE(jbresnah) When the image size is None it means that it is
+        # NOTE(jbresnah) When the image size is None it means that it is
         # not known.  In this case the only time we will raise an
         # exception is when there is no room left at all, thus we know
         # it will not fit.
@@ -439,6 +440,12 @@ class TestImagePropertyQuotas(test_utils.BaseTestCase):
         self.assertNotIn('foo', self.base_image.extra_properties)
         self.assertEqual('ham', self.base_image.extra_properties['spam'])
 
+    def test_invalid_quota_config_parameter(self):
+        self.config(user_storage_quota='foo')
+        location = {"url": "file:///fake.img.tar.gz", "metadata": {}}
+        self.assertRaises(exception.InvalidOptionValue,
+                          self.image.locations.append, location)
+
     def test_exceed_quota_during_patch_operation(self):
         self._quota_exceed_setup()
         self.image.extra_properties['frob'] = 'baz'
@@ -498,7 +505,7 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
     def test_replace_image_tag(self):
         self.config(image_tag_quota=1)
         self.image.tags = ['foo']
-        self.assertEqual(len(self.image.tags), 1)
+        self.assertEqual(1, len(self.image.tags))
 
     def test_replace_too_many_image_tags(self):
         self.config(image_tag_quota=0)
@@ -506,17 +513,17 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
         exc = self.assertRaises(exception.ImageTagLimitExceeded,
                                 setattr, self.image, 'tags', ['foo', 'bar'])
         self.assertIn('Attempted: 2, Maximum: 0', six.text_type(exc))
-        self.assertEqual(len(self.image.tags), 0)
+        self.assertEqual(0, len(self.image.tags))
 
     def test_replace_unlimited_image_tags(self):
         self.config(image_tag_quota=-1)
         self.image.tags = ['foo']
-        self.assertEqual(len(self.image.tags), 1)
+        self.assertEqual(1, len(self.image.tags))
 
     def test_add_image_tag(self):
         self.config(image_tag_quota=1)
         self.image.tags.add('foo')
-        self.assertEqual(len(self.image.tags), 1)
+        self.assertEqual(1, len(self.image.tags))
 
     def test_add_too_many_image_tags(self):
         self.config(image_tag_quota=1)
@@ -528,15 +535,15 @@ class TestImageTagQuotas(test_utils.BaseTestCase):
     def test_add_unlimited_image_tags(self):
         self.config(image_tag_quota=-1)
         self.image.tags.add('foo')
-        self.assertEqual(len(self.image.tags), 1)
+        self.assertEqual(1, len(self.image.tags))
 
     def test_remove_image_tag_while_over_quota(self):
         self.config(image_tag_quota=1)
         self.image.tags.add('foo')
-        self.assertEqual(len(self.image.tags), 1)
+        self.assertEqual(1, len(self.image.tags))
         self.config(image_tag_quota=0)
         self.image.tags.remove('foo')
-        self.assertEqual(len(self.image.tags), 0)
+        self.assertEqual(0, len(self.image.tags))
 
 
 class TestQuotaImageTagsProxy(test_utils.BaseTestCase):
@@ -568,15 +575,15 @@ class TestQuotaImageTagsProxy(test_utils.BaseTestCase):
                                                       'bar',
                                                       'baz',
                                                       'niz']))
-        self.assertEqual(len(proxy), 4)
+        self.assertEqual(4, len(proxy))
 
     def test_iter(self):
         items = set(['foo', 'bar', 'baz', 'niz'])
         proxy = glance.quota.QuotaImageTagsProxy(items.copy())
-        self.assertEqual(len(items), 4)
+        self.assertEqual(4, len(items))
         for item in proxy:
             items.remove(item)
-        self.assertEqual(len(items), 0)
+        self.assertEqual(0, len(items))
 
 
 class TestImageMemberQuotas(test_utils.BaseTestCase):
@@ -597,16 +604,16 @@ class TestImageMemberQuotas(test_utils.BaseTestCase):
 
         self.image_member_factory.new_image_member(self.image,
                                                    'fake_id')
-        self.base_image_member_factory.new_image_member\
-            .assert_called_once_with(self.image.base, 'fake_id')
+        nim = self.base_image_member_factory.new_image_member
+        nim .assert_called_once_with(self.image.base, 'fake_id')
 
     def test_new_image_member_unlimited_members(self):
         self.config(image_member_quota=-1)
 
         self.image_member_factory.new_image_member(self.image,
                                                    'fake_id')
-        self.base_image_member_factory.new_image_member\
-            .assert_called_once_with(self.image.base, 'fake_id')
+        nim = self.base_image_member_factory.new_image_member
+        nim.assert_called_once_with(self.image.base, 'fake_id')
 
     def test_new_image_member_too_many_members(self):
         self.config(image_member_quota=0)
@@ -640,7 +647,7 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         self.image.locations = [{"url": "file:///fake.img.tar.gz",
                                  "metadata": {}
                                  }]
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
 
     def test_replace_too_many_image_locations(self):
         self.config(image_location_quota=1)
@@ -655,20 +662,20 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         exc = self.assertRaises(exception.ImageLocationLimitExceeded,
                                 setattr, self.image, 'locations', locations)
         self.assertIn('Attempted: 3, Maximum: 1', six.text_type(exc))
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
 
     def test_replace_unlimited_image_locations(self):
         self.config(image_location_quota=-1)
         self.image.locations = [{"url": "file:///fake.img.tar.gz",
                                  "metadata": {}}
                                 ]
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
 
     def test_add_image_location(self):
         self.config(image_location_quota=1)
         location = {"url": "file:///fake.img.tar.gz", "metadata": {}}
         self.image.locations.append(location)
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
 
     def test_add_too_many_image_locations(self):
         self.config(image_location_quota=1)
@@ -683,13 +690,13 @@ class TestImageLocationQuotas(test_utils.BaseTestCase):
         self.config(image_location_quota=-1)
         location1 = {"url": "file:///fake1.img.tar.gz", "metadata": {}}
         self.image.locations.append(location1)
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
 
     def test_remove_image_location_while_over_quota(self):
         self.config(image_location_quota=1)
         location1 = {"url": "file:///fake1.img.tar.gz", "metadata": {}}
         self.image.locations.append(location1)
-        self.assertEqual(len(self.image.locations), 1)
+        self.assertEqual(1, len(self.image.locations))
         self.config(image_location_quota=0)
         self.image.locations.remove(location1)
-        self.assertEqual(len(self.image.locations), 0)
+        self.assertEqual(0, len(self.image.locations))
