@@ -12,18 +12,17 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-import mock
 import uuid
 
 import glance_store
+import mock
 import six
 import webob
 
+import glance.api.policy
 import glance.api.v2.image_data
 from glance.common import exception
 from glance.common import wsgi
-
 from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
@@ -110,13 +109,25 @@ class TestImagesController(base.StoreClearingUnitTest):
                                       'metadata': {}, 'status': 'active'}])
         self.image_repo.result = image
         image = self.controller.download(request, unit_test_utils.UUID1)
-        self.assertEqual(image.image_id, 'abcd')
+        self.assertEqual('abcd', image.image_id)
+
+    def test_download_deactivated(self):
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd',
+                          status='deactivated',
+                          locations=[{'url': 'http://example.com/image',
+                                      'metadata': {}, 'status': 'active'}])
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.download,
+                          request, str(uuid.uuid4()))
 
     def test_download_no_location(self):
+        # NOTE(mclaren): NoContent will be raised by the ResponseSerializer
+        # That's tested below.
         request = unit_test_utils.get_fake_request()
         self.image_repo.result = FakeImage('abcd')
-        self.assertRaises(webob.exc.HTTPNoContent, self.controller.download,
-                          request, unit_test_utils.UUID2)
+        image = self.controller.download(request, unit_test_utils.UUID2)
+        self.assertEqual('abcd', image.image_id)
 
     def test_download_non_existent_image(self):
         request = unit_test_utils.get_fake_request()
@@ -130,7 +141,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.download,
                           request, str(uuid.uuid4()))
 
-    def test_download_get_image_location_forbidden(self):
+    def test_download_ok_when_get_image_location_forbidden(self):
         class ImageLocations(object):
             def __len__(self):
                 raise exception.Forbidden()
@@ -139,16 +150,16 @@ class TestImagesController(base.StoreClearingUnitTest):
         image = FakeImage('abcd')
         self.image_repo.result = image
         image.locations = ImageLocations()
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.download,
-                          request, str(uuid.uuid4()))
+        image = self.controller.download(request, unit_test_utils.UUID1)
+        self.assertEqual('abcd', image.image_id)
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd')
         self.image_repo.result = image
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
-        self.assertEqual(image.data, 'YYYY')
-        self.assertEqual(image.size, 4)
+        self.assertEqual('YYYY', image.data)
+        self.assertEqual(4, image.size)
 
     def test_upload_status(self):
         request = unit_test_utils.get_fake_request()
@@ -158,21 +169,21 @@ class TestImagesController(base.StoreClearingUnitTest):
 
         def read_data():
             insurance['called'] = True
-            self.assertEqual(self.image_repo.saved_image.status, 'saving')
+            self.assertEqual('saving', self.image_repo.saved_image.status)
             yield 'YYYY'
 
         self.controller.upload(request, unit_test_utils.UUID2,
                                read_data(), None)
         self.assertTrue(insurance['called'])
-        self.assertEqual(self.image_repo.saved_image.status,
-                         'modified-by-fake')
+        self.assertEqual('modified-by-fake',
+                         self.image_repo.saved_image.status)
 
     def test_upload_no_size(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd')
         self.image_repo.result = image
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', None)
-        self.assertEqual(image.data, 'YYYY')
+        self.assertEqual('YYYY', image.data)
         self.assertIsNone(image.size)
 
     def test_upload_invalid(self):
@@ -183,6 +194,23 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYY', 4)
 
+    def test_upload_with_expired_token(self):
+        def side_effect(image, from_state=None):
+            if from_state == 'saving':
+                raise exception.NotAuthenticated()
+
+        mocked_save = mock.Mock(side_effect=side_effect)
+        mocked_delete = mock.Mock()
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd')
+        image.delete = mocked_delete
+        self.image_repo.result = image
+        self.image_repo.save = mocked_save
+        self.assertRaises(webob.exc.HTTPUnauthorized, self.controller.upload,
+                          request, unit_test_utils.UUID1, 'YYYY', 4)
+        self.assertEqual(3, mocked_save.call_count)
+        mocked_delete.assert_called_once_with()
+
     def test_upload_non_existent_image_during_save_initiates_deletion(self):
         def fake_save_not_found(self):
             raise exception.NotFound()
@@ -307,7 +335,7 @@ class TestImagesController(base.StoreClearingUnitTest):
             'event_type': "image.prepare",
             'payload': prepare_payload,
         }
-        self.assertEqual(len(output_log), 3)
+        self.assertEqual(3, len(output_log))
         prepare_updated_at = output_log[0]['payload']['updated_at']
         del output_log[0]['payload']['updated_at']
         self.assertTrue(prepare_updated_at <= output['meta']['updated_at'])
@@ -324,7 +352,7 @@ class TestImagesController(base.StoreClearingUnitTest):
             'event_type': "image.upload",
             'payload': upload_payload,
         }
-        self.assertEqual(len(output_log), 3)
+        self.assertEqual(3, len(output_log))
         self.assertEqual(output_log[1], upload_log)
 
     def _test_upload_download_activate_notification(self):
@@ -338,7 +366,7 @@ class TestImagesController(base.StoreClearingUnitTest):
             'event_type': "image.activate",
             'payload': activate_payload,
         }
-        self.assertEqual(len(output_log), 3)
+        self.assertEqual(3, len(output_log))
         self.assertEqual(output_log[2], activate_log)
 
     def test_restore_image_when_upload_failed(self):
@@ -349,7 +377,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPServiceUnavailable,
                           self.controller.upload,
                           request, unit_test_utils.UUID2, 'ZZZ', 3)
-        self.assertEqual(self.image_repo.saved_image.status, 'queued')
+        self.assertEqual('queued', self.image_repo.saved_image.status)
 
 
 class TestImageDataDeserializer(test_utils.BaseTestCase):
@@ -365,7 +393,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.headers['Content-Length'] = 3
         output = self.deserializer.upload(request)
         data = output.pop('data')
-        self.assertEqual(data.read(), 'YYY')
+        self.assertEqual('YYY', data.read())
         expected = {'size': 3}
         self.assertEqual(expected, output)
 
@@ -377,7 +405,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.body_file = six.StringIO('YYY')
         output = self.deserializer.upload(request)
         data = output.pop('data')
-        self.assertEqual(data.read(), 'YYY')
+        self.assertEqual('YYY', data.read())
         expected = {'size': None}
         self.assertEqual(expected, output)
 
@@ -390,7 +418,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.headers['Content-Length'] = 3
         output = self.deserializer.upload(request)
         data = output.pop('data')
-        self.assertEqual(data.read(), 'YYY')
+        self.assertEqual('YYY', data.read())
         expected = {'size': 3}
         self.assertEqual(expected, output)
 
@@ -404,7 +432,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.headers['Content-Length'] = 4
         output = self.deserializer.upload(request)
         data = output.pop('data')
-        self.assertEqual(data.read(), 'YYY')
+        self.assertEqual('YYY', data.read())
         expected = {'size': 4}
         self.assertEqual(expected, output)
 
@@ -475,22 +503,78 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
                           self.serializer.download,
                           response, image)
 
-    def test_download_not_found(self):
-        """Test image download returns HTTPNotFound.
+    def test_download_no_content(self):
+        """Test image download returns HTTPNoContent
 
-        Make sure that serializer returns 404 not found error in case of
-        image is not available at specified location.
+        Make sure that serializer returns 204 no content error in case of
+        image data is not available at specified location.
         """
         with mock.patch.object(glance.api.policy.ImageProxy,
                                'get_data') as mock_get_data:
-            mock_get_data.side_effect = glance_store.NotFound()
+            mock_get_data.side_effect = glance_store.NotFound(image="image")
 
             request = wsgi.Request.blank('/')
             response = webob.Response()
             response.request = request
             image = FakeImage(size=3, data=iter('ZZZ'))
             image.get_data = mock_get_data
-            self.assertRaises(webob.exc.HTTPNotFound,
+            self.assertRaises(webob.exc.HTTPNoContent,
+                              self.serializer.download,
+                              response, image)
+
+    def test_download_service_unavailable(self):
+        """Test image download returns HTTPServiceUnavailable."""
+        with mock.patch.object(glance.api.policy.ImageProxy,
+                               'get_data') as mock_get_data:
+            mock_get_data.side_effect = glance_store.RemoteServiceUnavailable()
+
+            request = wsgi.Request.blank('/')
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=iter('ZZZ'))
+            image.get_data = mock_get_data
+            self.assertRaises(webob.exc.HTTPServiceUnavailable,
+                              self.serializer.download,
+                              response, image)
+
+    def test_download_store_get_not_support(self):
+        """Test image download returns HTTPBadRequest.
+
+        Make sure that serializer returns 400 bad request error in case of
+        getting images from this store is not supported at specified location.
+        """
+        with mock.patch.object(glance.api.policy.ImageProxy,
+                               'get_data') as mock_get_data:
+            mock_get_data.side_effect = glance_store.StoreGetNotSupported()
+
+            request = wsgi.Request.blank('/')
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=iter('ZZZ'))
+            image.get_data = mock_get_data
+            self.assertRaises(webob.exc.HTTPBadRequest,
+                              self.serializer.download,
+                              response, image)
+
+    def test_download_store_random_get_not_support(self):
+        """Test image download returns HTTPBadRequest.
+
+        Make sure that serializer returns 400 bad request error in case of
+        getting randomly images from this store is not supported at
+        specified location.
+        """
+        with mock.patch.object(glance.api.policy.ImageProxy,
+                               'get_data') as m_get_data:
+            err = glance_store.StoreRandomGetNotSupported(offset=0,
+                                                          chunk_size=0)
+            m_get_data.side_effect = err
+
+            request = wsgi.Request.blank('/')
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=iter('ZZZ'))
+            image.get_data = m_get_data
+            self.assertRaises(webob.exc.HTTPBadRequest,
                               self.serializer.download,
                               response, image)
 
