@@ -15,6 +15,7 @@
 
 """Common utilities used in testing"""
 
+import BaseHTTPServer
 import errno
 import functools
 import os
@@ -24,21 +25,22 @@ import socket
 import subprocess
 
 import fixtures
-from oslo.config import cfg
+from oslo.serialization import jsonutils
+from oslo_config import cfg
+from oslo_utils import timeutils
+from oslotest import moxstubout
 import six
-import stubout
 import testtools
 import webob
 
 from glance.common import config
 from glance.common import exception
 from glance.common import property_utils
+from glance.common import utils
 from glance.common import wsgi
 from glance import context
 from glance.db.sqlalchemy import api as db_api
 from glance.db.sqlalchemy import models as db_models
-from glance.openstack.common import jsonutils
-from glance.openstack.common import timeutils
 
 CONF = cfg.CONF
 
@@ -48,19 +50,23 @@ class BaseTestCase(testtools.TestCase):
     def setUp(self):
         super(BaseTestCase, self).setUp()
 
-        #NOTE(bcwaldon): parse_args has to be called to register certain
+        # NOTE(bcwaldon): parse_args has to be called to register certain
         # command-line options - specifically we need config_dir for
         # the following policy tests
         config.parse_args(args=[])
         self.addCleanup(CONF.reset)
-        self.stubs = stubout.StubOutForTesting()
+        mox_fixture = self.useFixture(moxstubout.MoxStubout())
+        self.stubs = mox_fixture.stubs
         self.stubs.Set(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
         self.test_dir = self.useFixture(fixtures.TempDir()).path
+        self.conf_dir = os.path.join(self.test_dir, 'etc')
+        utils.safe_mkdirs(self.conf_dir)
+        self.set_policy()
 
-    def tearDown(self):
-        self.stubs.UnsetAll()
-        self.stubs.SmartUnsetAll()
-        super(BaseTestCase, self).tearDown()
+    def set_policy(self):
+        conf_file = "policy.json"
+        self.policy_file = self._copy_data_file(conf_file, self.conf_dir)
+        self.config(policy_file=self.policy_file, group='oslo_policy')
 
     def set_property_protections(self, use_policies=False):
         self.unset_property_protections()
@@ -273,13 +279,13 @@ def execute(cmd,
     path_ext = [os.path.join(os.getcwd(), 'bin')]
 
     # Also jack in the path cmd comes from, if it's absolute
-    executable = cmd.split()[0]
+    args = shlex.split(cmd)
+    executable = args[0]
     if os.path.isabs(executable):
         path_ext.append(os.path.dirname(executable))
 
     env['PATH'] = ':'.join(path_ext) + ':' + env['PATH']
-    process = subprocess.Popen(cmd,
-                               shell=True,
+    process = subprocess.Popen(args,
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
@@ -294,11 +300,11 @@ def execute(cmd,
         exitcode = 0
 
     if exitcode != expected_exitcode and raise_error:
-        msg = "Command %(cmd)s did not succeed. Returned an exit "\
-              "code of %(exitcode)d."\
-              "\n\nSTDOUT: %(out)s"\
-              "\n\nSTDERR: %(err)s" % {'cmd': cmd, 'exitcode': exitcode,
-                                       'out': out, 'err': err}
+        msg = ("Command %(cmd)s did not succeed. Returned an exit "
+               "code of %(exitcode)d."
+               "\n\nSTDOUT: %(out)s"
+               "\n\nSTDERR: %(err)s" % {'cmd': cmd, 'exitcode': exitcode,
+                                        'out': out, 'err': err})
         if context:
             msg += "\n\nCONTEXT: %s" % context
         raise RuntimeError(msg)
@@ -404,6 +410,41 @@ def minimal_add_command(port, name, suffix='', public=True):
             " name=%s %s" % (port, visibility, name, suffix))
 
 
+def start_http_server(image_id, image_data):
+    def _get_http_handler_class(fixture):
+        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+            def do_GET(self):
+                self.send_response(200)
+                self.send_header('Content-Length', str(len(fixture)))
+                self.end_headers()
+                self.wfile.write(fixture)
+                return
+
+            def do_HEAD(self):
+                self.send_response(200)
+                self.send_header('Content-Length', str(len(fixture)))
+                self.end_headers()
+                return
+
+            def log_message(self, *args, **kwargs):
+                # Override this method to prevent debug output from going
+                # to stderr during testing
+                return
+
+        return StaticHTTPRequestHandler
+
+    server_address = ('127.0.0.1', 0)
+    handler_class = _get_http_handler_class(image_data)
+    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    port = httpd.socket.getsockname()[1]
+
+    pid = os.fork()
+    if pid == 0:
+        httpd.serve_forever()
+    else:
+        return pid, port
+
+
 class RegistryAPIMixIn(object):
 
     def create_fixtures(self):
