@@ -12,6 +12,7 @@ function usage {
   echo "  -f, --force                 Force a clean re-build of the virtual environment. Useful when dependencies have been added."
   echo "  -u, --update                Update the virtual environment with any newer package versions"
   echo "  -p, --pep8                  Just run PEP8 and HACKING compliance check"
+  echo "  -8, --pep8-only-changed     Just run PEP8 and HACKING compliance check on files changed since HEAD~1"
   echo "  -P, --no-pep8               Don't run static code checks"
   echo "  -c, --coverage              Generate coverage report"
   echo "  -d, --debug                 Run tests with testtools instead of testr. This allows you to use the debugger."
@@ -23,7 +24,7 @@ function usage {
   echo "  --tools-path <dir>          Location of the tools directory"
   echo "                               Default: \$(pwd)"
   echo "  --concurrency <concurrency> How many processes to use when running the tests. A value of 0 autodetects concurrency from your CPU count"
-  echo "                               Default: 1"
+  echo "                               Default: 0"
   echo ""
   echo "Note: with no options specified, the script will try to run the tests in a virtual environment,"
   echo "      If no virtualenv is found, the script will ask if you would like to create one.  If you "
@@ -42,6 +43,7 @@ function process_options {
       -f|--force) force=1;;
       -u|--update) update=1;;
       -p|--pep8) just_pep8=1;;
+      -8|--pep8-only-changed) just_pep8_changed=1;;
       -P|--no-pep8) no_pep8=1;;
       -c|--coverage) coverage=1;;
       -d|--debug) debug=1;;
@@ -81,11 +83,12 @@ testrargs=
 testropts=
 wrapper=""
 just_pep8=0
+just_pep8_changed=0
 no_pep8=0
 coverage=0
 debug=0
 update=0
-concurrency=1
+concurrency=0
 
 LANG=en_US.UTF-8
 LANGUAGE=en_US:en
@@ -166,17 +169,21 @@ function copy_subunit_log {
   cp $LOGNAME subunit.log
 }
 
+function warn_on_flake8_without_venv {
+  if [ $never_venv -eq 1 ]; then
+    echo "**WARNING**:"
+    echo "Running flake8 without virtual env may miss OpenStack HACKING detection"
+  fi
+}
+
 function run_pep8 {
   echo "Running flake8 ..."
-  if [ $never_venv -eq 1 ]; then
-      echo "**WARNING**:"
-      echo "Running flake8 without virtual env may miss OpenStack HACKING detection"
-  fi
+  warn_on_flake8_without_venv
   bash -c "${wrapper} flake8"
 }
 
 
-TESTRTESTS="python -m glance.openstack.common.lockutils python setup.py testr"
+TESTRTESTS="lockutils-wrapper python setup.py testr"
 
 if [ $never_venv -eq 0 ]
 then
@@ -218,6 +225,19 @@ if [ $just_pep8 -eq 1 ]; then
     exit
 fi
 
+if [ $just_pep8_changed -eq 1 ]; then
+    # NOTE(gilliard) We want use flake8 to check the entirety of every file that has
+    # a change in it. Unfortunately the --filenames argument to flake8 only accepts
+    # file *names* and there are no files named (eg) "nova/compute/manager.py".  The
+    # --diff argument behaves surprisingly as well, because although you feed it a
+    # diff, it actually checks the file on disk anyway.
+    files=$(git diff --name-only HEAD~1 | tr '\n' ' ')
+    echo "Running flake8 on ${files}"
+    warn_on_flake8_without_venv
+    bash -c "diff -u --from-file /dev/null ${files} | ${wrapper} flake8 --diff"
+    exit
+fi
+
 run_tests
 
 # NOTE(sirp): we only want to run pep8 when we're running the full-test suite,
