@@ -42,18 +42,17 @@
 """Display a subunit stream through a colorized unittest test runner."""
 
 import heapq
-import subunit
 import sys
 import unittest
 
+import subunit
 import testtools
 
 
 class _AnsiColorizer(object):
-    """
-    A colorizer is an object that loosely wraps around a stream, allowing
-    callers to write text to the stream in a particular color.
+    """A colorizer is an object that loosely wraps around a stream.
 
+    That allows callers to write text to the stream in a particular color.
     Colorizer classes must implement C{supported()} and C{write(text, color)}.
     """
     _colors = dict(black=30, red=31, green=32, yellow=33,
@@ -62,10 +61,11 @@ class _AnsiColorizer(object):
     def __init__(self, stream):
         self.stream = stream
 
-    def supported(cls, stream=sys.stdout):
-        """
-        A class method that returns True if the current platform supports
-        coloring terminal output using this method. Returns False otherwise.
+    @staticmethod
+    def supported(stream=sys.stdout):
+        """Method that checks if the current terminal supports coloring.
+
+        Returns True or False.
         """
         if not stream.isatty():
             return False  # auto color only on TTYs
@@ -83,24 +83,21 @@ class _AnsiColorizer(object):
             except Exception:
                 # guess false in case of error
                 return False
-    supported = classmethod(supported)
 
     def write(self, text, color):
-        """
-        Write the given text to the stream in the given color.
+        """Write the given text to the stream in the given color.
 
         @param text: Text to be written to the stream.
 
         @param color: A string label for a color. e.g. 'red', 'white'.
+
         """
         color = self._colors[color]
         self.stream.write('\x1b[%s;1m%s\x1b[0m' % (color, text))
 
 
 class _Win32Colorizer(object):
-    """
-    See _AnsiColorizer docstring.
-    """
+    """See _AnsiColorizer docstring."""
     def __init__(self, stream):
         import win32console
         red, green, blue, bold = (win32console.FOREGROUND_RED,
@@ -121,7 +118,8 @@ class _Win32Colorizer(object):
             'white': red | green | blue | bold
         }
 
-    def supported(cls, stream=sys.stdout):
+    @staticmethod
+    def supported(stream=sys.stdout):
         try:
             import win32console
             screenBuffer = win32console.GetStdHandle(
@@ -138,7 +136,6 @@ class _Win32Colorizer(object):
             return False
         else:
             return True
-    supported = classmethod(supported)
 
     def write(self, text, color):
         color = self._colors[color]
@@ -148,15 +145,13 @@ class _Win32Colorizer(object):
 
 
 class _NullColorizer(object):
-    """
-    See _AnsiColorizer docstring.
-    """
+    """See _AnsiColorizer docstring."""
     def __init__(self, stream):
         self.stream = stream
 
-    def supported(cls, stream=sys.stdout):
+    @staticmethod
+    def supported(stream=sys.stdout):
         return True
-    supported = classmethod(supported)
 
     def write(self, text, color):
         self.stream.write(text)
